@@ -1,4 +1,7 @@
+#include <afsconfig.h>
 #include <afs/param.h>
+#include <roken.h>
+
 #include <afs/stds.h>
 
 #include <windows.h>
@@ -7,6 +10,7 @@
 #include <stdlib.h>
 #include <malloc.h>
 #include <string.h>
+#include <cm_nls.h>
 
 #include <WINNT/afsreg.h>
 #include "afsd.h"
@@ -14,19 +18,20 @@
 
 #ifdef AFS_FREELANCE_CLIENT
 #include "cm_freelance.h"
-#include "stdio.h"
+#include <stdio.h>
+#define STRSAFE_NO_DEPRECATE
+#include <strsafe.h>
 
 extern void afsi_log(char *pattern, ...);
 
-int cm_noLocalMountPoints;
+static unsigned int cm_noLocalMountPoints = 0;
 char * cm_FakeRootDir = NULL;
 int cm_fakeDirSize = 0;
-int cm_fakeDirCallback=0;
-int cm_fakeGettingCallback=0;
-cm_localMountPoint_t* cm_localMountPoints;
+static cm_localMountPoint_t* cm_localMountPoints;
 osi_mutex_t cm_Freelance_Lock;
-int cm_localMountPointChangeFlag = 0;
+static int cm_localMountPointChangeFlag = 0;
 int cm_freelanceEnabled = 1;
+int cm_freelanceImportCellServDB = 0;
 time_t FakeFreelanceModTime = 0x3b49f6e2;
 
 static int freelance_ShutdownFlag = 0;
@@ -38,7 +43,7 @@ void cm_InitFakeRootDir();
 void cm_FreelanceChangeNotifier(void * parmp) {
     HKEY   hkFreelance = 0;
 
-    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
+    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                       AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance",
                       0,
                       KEY_NOTIFY,
@@ -52,7 +57,7 @@ void cm_FreelanceChangeNotifier(void * parmp) {
     }
 
     while ( TRUE ) {
-    /* check hFreelanceChangeEvent to see if it is set. 
+    /* check hFreelanceChangeEvent to see if it is set.
      * if so, call cm_noteLocalMountPointChange()
      */
         if (RegNotifyChangeKeyValue( hkFreelance,   /* hKey */
@@ -69,13 +74,13 @@ void cm_FreelanceChangeNotifier(void * parmp) {
 
         if (WaitForSingleObject(hFreelanceChangeEvent, INFINITE) == WAIT_OBJECT_0)
         {
-            if (freelance_ShutdownFlag == 1) {     
-                RegCloseKey(hkFreelance);          
+            if (freelance_ShutdownFlag == 1) {
+                RegCloseKey(hkFreelance);
                 CloseHandle(hFreelanceChangeEvent);
-                hFreelanceChangeEvent = 0;         
-                return;                            
-            }                                      
-            cm_noteLocalMountPointChange();
+                hFreelanceChangeEvent = 0;
+                return;
+            }
+            cm_noteLocalMountPointChange(FALSE);
         }
     }
 }
@@ -83,7 +88,7 @@ void cm_FreelanceChangeNotifier(void * parmp) {
 void cm_FreelanceSymlinkChangeNotifier(void * parmp) {
     HKEY   hkFreelance = 0;
 
-    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
+    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                       AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance\\Symlinks",
                       0,
                       KEY_NOTIFY,
@@ -97,7 +102,7 @@ void cm_FreelanceSymlinkChangeNotifier(void * parmp) {
     }
 
     while ( TRUE ) {
-    /* check hFreelanceSymlinkChangeEvent to see if it is set. 
+    /* check hFreelanceSymlinkChangeEvent to see if it is set.
      * if so, call cm_noteLocalMountPointSymlinkChange()
      */
         if (RegNotifyChangeKeyValue( hkFreelance,   /* hKey */
@@ -114,26 +119,48 @@ void cm_FreelanceSymlinkChangeNotifier(void * parmp) {
 
         if (WaitForSingleObject(hFreelanceSymlinkChangeEvent, INFINITE) == WAIT_OBJECT_0)
         {
-            if (freelance_ShutdownFlag == 1) {     
-                RegCloseKey(hkFreelance);          
+            if (freelance_ShutdownFlag == 1) {
+                RegCloseKey(hkFreelance);
                 CloseHandle(hFreelanceSymlinkChangeEvent);
-                hFreelanceSymlinkChangeEvent = 0;         
-                return;                            
-            }                                      
-            cm_noteLocalMountPointChange();
+                hFreelanceSymlinkChangeEvent = 0;
+                return;
+            }
+            cm_noteLocalMountPointChange(FALSE);
         }
     }
 }
 
-void                                          
-cm_FreelanceShutdown(void)                    
-{                                             
-    freelance_ShutdownFlag = 1;               
-    if (hFreelanceChangeEvent != 0)           
-        thrd_SetEvent(hFreelanceChangeEvent); 
-    if (hFreelanceSymlinkChangeEvent != 0)           
-        thrd_SetEvent(hFreelanceSymlinkChangeEvent); 
-}                                             
+void
+cm_FreelanceShutdown(void)
+{
+    freelance_ShutdownFlag = 1;
+    if (hFreelanceChangeEvent != 0)
+        thrd_SetEvent(hFreelanceChangeEvent);
+    if (hFreelanceSymlinkChangeEvent != 0)
+        thrd_SetEvent(hFreelanceSymlinkChangeEvent);
+}
+
+static long
+cm_FreelanceAddCSDBMountPoints(void *rockp, char *cellNamep)
+{
+    char szCellName[CELL_MAXNAMELEN+1] = ".";
+
+    cm_FsStrCpy( &szCellName[1], CELL_MAXNAMELEN, cellNamep);
+    /* create readonly mount point */
+    cm_FreelanceAddMount( cellNamep, cellNamep, "root.cell", 0, NULL);
+
+    /* create read/write mount point */
+    cm_FreelanceAddMount( szCellName, szCellName, "root.cell", 1, NULL);
+
+    return 0;
+}
+
+void
+cm_FreelanceImportCellServDB(void)
+{
+    cm_EnumerateCellRegistry( TRUE, cm_FreelanceAddCSDBMountPoints, NULL);
+    cm_EnumerateCellFile( TRUE, cm_FreelanceAddCSDBMountPoints, NULL);
+}
 
 void cm_InitFreelance() {
     thread_t phandle;
@@ -141,6 +168,8 @@ void cm_InitFreelance() {
 
     lock_InitializeMutex(&cm_Freelance_Lock, "Freelance Lock", LOCK_HIERARCHY_FREELANCE_GLOBAL);
 
+    lock_ObtainMutex(&cm_Freelance_Lock);
+
     // yj: first we make a call to cm_initLocalMountPoints
     // to read all the local mount points from the registry
     cm_InitLocalMountPoints();
@@ -148,7 +177,12 @@ void cm_InitFreelance() {
     // then we make a call to InitFakeRootDir to create
     // a fake root directory based on the local mount points
     cm_InitFakeRootDir();
+
+    // increment the fakeDirVersion to force status updates for
+    // all cached Freelance objects
+    cm_data.fakeDirVersion++;
     // --- end of yj code
+    lock_ReleaseMutex(&cm_Freelance_Lock);
 
     /* Start the registry monitor */
     phandle = thrd_Create(NULL, 65536, (ThreadFunc) cm_FreelanceChangeNotifier,
@@ -163,7 +197,7 @@ void cm_InitFreelance() {
 }
 
 /* yj: Initialization of the fake root directory */
-/* to be called while holding freelance lock unless during init. */
+/* to be called while holding freelance lock. */
 void cm_InitFakeRootDir() {
     int i, t1, t2;
     char* currentPos;
@@ -183,7 +217,7 @@ void cm_InitFakeRootDir() {
     // 4. the first chunk of all subsequent pages are used
     //    for page header stuff
     // 5. a max of CM_DIR_EPP entries are allowed per page
-    // 6. each entry takes 1 or more chunks, depending on 
+    // 6. each entry takes 1 or more chunks, depending on
     //    the size of the mount point string, as determined
     //    by cm_NameEntries
     // 7. each chunk is CM_DIR_CHUNKSIZE bytes
@@ -192,11 +226,14 @@ void cm_InitFakeRootDir() {
     int curChunk = 13;	// chunks 0 - 12 are used for header stuff
                         // of the first page in the directory
     int curPage = 0;
-    int curDirEntry = 0;
+    unsigned int curDirEntry = 0;
     int curDirEntryInPage = 0;
     int sizeOfCurEntry;
     int dirSize;
 
+    /* Increment the fake Uniquifier */
+    cm_data.fakeUnique++;
+
     /* Reserve 2 directory chunks for "." and ".." */
     curChunk += 2;
 
@@ -217,11 +254,11 @@ void cm_InitFakeRootDir() {
     if (cm_fakeDirSize != dirSize) {
         if (cm_FakeRootDir)
             free(cm_FakeRootDir);
-        cm_FakeRootDir = malloc(dirSize);
+        cm_FakeRootDir = calloc(dirSize, 1);
         cm_fakeDirSize = dirSize;
     }
 
-    // yj: when we get here, we've figured out how much memory we need and 
+    // yj: when we get here, we've figured out how much memory we need and
     // allocated the appropriate space for it. we now prceed to fill
     // it up with entries.
     curPage = 0;
@@ -233,16 +270,16 @@ void cm_InitFakeRootDir() {
     fakeEntry.flag = 1;
     fakeEntry.length = 0;
     fakeEntry.next = 0;
-    fakeEntry.fid.unique = htonl(1);
+    fakeEntry.fid.unique = htonl(1 + cm_data.fakeUnique);
 
     // the first page is special, it uses fakeDirHeader instead of fakePageHeader
     // we fill up the page with dirEntries that belong there and we make changes
     // to the fakeDirHeader.header.freeBitmap along the way. Then when we're done
-    // filling up the dirEntries in this page, we copy the fakeDirHeader into 
+    // filling up the dirEntries in this page, we copy the fakeDirHeader into
     // the top of the page.
 
     // init the freeBitmap array
-    for (i=0; i<8; i++) 
+    for (i=0; i<8; i++)
         fakeDirHeader.header.freeBitmap[i]=0;
 
     fakeDirHeader.header.freeBitmap[0] = 0xff;
@@ -268,13 +305,15 @@ void cm_InitFakeRootDir() {
     // 2. we have less than CM_DIR_EPP entries in page 0
     // 3. we're not out of chunks in page 0
 
-    while( (curDirEntry<cm_noLocalMountPoints) && 
+    while( (curDirEntry<cm_noLocalMountPoints) &&
            (curDirEntryInPage < CM_DIR_EPP) &&
-           (curChunk + cm_NameEntries((cm_localMountPoints+curDirEntry)->namep, 0) <= CPP)) 
-    {       
+           (curChunk + cm_NameEntries((cm_localMountPoints+curDirEntry)->namep, 0) <= CPP))
+    {
 
         noChunks = cm_NameEntries((cm_localMountPoints+curDirEntry)->namep, 0);
-        fakeEntry.fid.vnode = htonl(curDirEntry + 2);
+        /* enforce the rule that only directories have odd vnode values */
+        fakeEntry.fid.vnode = htonl((curDirEntry + 1) * 2);
+        fakeEntry.fid.unique = htonl(curDirEntry + 1 + cm_data.fakeUnique);
         currentPos = cm_FakeRootDir + curPage * CM_DIR_PAGESIZE + curChunk * CM_DIR_CHUNKSIZE;
 
         memcpy(currentPos, &fakeEntry, CM_DIR_CHUNKSIZE);
@@ -300,7 +339,7 @@ void cm_InitFakeRootDir() {
     while (curDirEntry<cm_noLocalMountPoints) {
         // setup a new page
         curChunk = 1;			// the zeroth chunk is reserved for page header
-        curDirEntryInPage = 0; 
+        curDirEntryInPage = 0;
         for (i=0; i<8; i++) {
             fakePageHeader.freeBitmap[i]=0;
         }
@@ -316,7 +355,9 @@ void cm_InitFakeRootDir() {
             // add an entry to this page
 
             noChunks = cm_NameEntries((cm_localMountPoints+curDirEntry)->namep, 0);
-            fakeEntry.fid.vnode=htonl(curDirEntry+2);
+            /* enforce the rule that only directories have odd vnode values */
+            fakeEntry.fid.vnode = htonl((curDirEntry + 1) * 2);
+            fakeEntry.fid.unique = htonl(curDirEntry + 1 + cm_data.fakeUnique);
             currentPos = cm_FakeRootDir + curPage * CM_DIR_PAGESIZE + curChunk * CM_DIR_CHUNKSIZE;
             memcpy(currentPos, &fakeEntry, CM_DIR_CHUNKSIZE);
             strcpy(currentPos + 12, (cm_localMountPoints+curDirEntry)->namep);
@@ -335,28 +376,38 @@ void cm_InitFakeRootDir() {
     }
 
     // we know the fakeDir is setup properly, so we claim that we have callback
-    osi_Log0(afsd_logp,"cm_InitFakeRootDir fakeDirCallback=1");
-    cm_fakeDirCallback=1;
+    osi_Log0(afsd_logp,"cm_InitFakeRootDir completed!");
 
     // when we get here, we've set up everything! done!
 }
 
 int cm_FakeRootFid(cm_fid_t *fidp)
 {
-    cm_SetFid(fidp, 
+    cm_SetFid(fidp,
               AFS_FAKE_ROOT_CELL_ID,            /* root cell */
               AFS_FAKE_ROOT_VOL_ID,            /* root.afs ? */
               1, 1);
     return 0;
 }
-  
+
 /* called directly from ioctl */
 /* called while not holding freelance lock */
-int cm_noteLocalMountPointChange(void) {
-    lock_ObtainMutex(&cm_Freelance_Lock);
+int cm_noteLocalMountPointChange(afs_int32 locked) {
+    if (!locked)
+        lock_ObtainMutex(&cm_Freelance_Lock);
     cm_data.fakeDirVersion++;
     cm_localMountPointChangeFlag = 1;
-    lock_ReleaseMutex(&cm_Freelance_Lock);
+
+    if (RDR_Initialized) {
+        cm_fid_t fid;
+        cm_FakeRootFid(&fid);
+        RDR_InvalidateObject( fid.cell, fid.volume, fid.vnode, fid.unique, fid.hash,
+                              CM_SCACHETYPE_DIRECTORY,
+                              AFS_INVALIDATE_DATA_VERSION);
+    }
+
+    if (!locked)
+        lock_ReleaseMutex(&cm_Freelance_Lock);
     return 1;
 }
 
@@ -371,9 +422,12 @@ int cm_clearLocalMountPointChange() {
 
 int cm_reInitLocalMountPoints() {
     cm_fid_t aFid;
-    int i, hash;
+    unsigned int i, hash;
     cm_scache_t *scp, **lscpp, *tscp;
-	
+    cm_req_t req;
+
+    cm_InitReq(&req);
+
     osi_Log0(afsd_logp,"----- freelance reinitialization starts ----- ");
 
     // first we invalidate all the SCPs that were created
@@ -381,11 +435,13 @@ int cm_reInitLocalMountPoints() {
 
     osi_Log0(afsd_logp,"Invalidating local mount point scp...  ");
 
-
     lock_ObtainWrite(&cm_scacheLock);
     lock_ObtainMutex(&cm_Freelance_Lock);  /* always scache then freelance lock */
-    for (i=1; i<=cm_noLocalMountPoints; i++) {
-        cm_SetFid(&aFid, AFS_FAKE_ROOT_CELL_ID, AFS_FAKE_ROOT_VOL_ID, i, 1);
+    for (i=0; i<=cm_noLocalMountPoints; i++) {
+        if (i == 0)
+            cm_SetFid(&aFid, AFS_FAKE_ROOT_CELL_ID, AFS_FAKE_ROOT_VOL_ID, 1, 1);
+        else
+            cm_SetFid(&aFid, AFS_FAKE_ROOT_CELL_ID, AFS_FAKE_ROOT_VOL_ID, i*2, i);
         hash = CM_SCACHE_HASH(&aFid);
         for (scp=cm_data.scacheHashTablep[hash]; scp; scp=scp->nextp) {
             if (scp != cm_data.rootSCachep && cm_FidCmp(&scp->fid, &aFid) == 0) {
@@ -398,8 +454,8 @@ int cm_reInitLocalMountPoints() {
 
                 // take the scp out of the hash
                 lock_ObtainWrite(&cm_scacheLock);
-                for (lscpp = &cm_data.scacheHashTablep[hash], tscp = cm_data.scacheHashTablep[hash]; 
-                     tscp; 
+                for (lscpp = &cm_data.scacheHashTablep[hash], tscp = cm_data.scacheHashTablep[hash];
+                     tscp;
                      lscpp = &tscp->nextp, tscp = tscp->nextp) {
                     if (tscp == scp) {
                         *lscpp = scp->nextp;
@@ -412,6 +468,11 @@ int cm_reInitLocalMountPoints() {
                 lock_ReleaseWrite(&scp->rw);
                 lock_ReleaseWrite(&cm_scacheLock);
                 cm_CallbackNotifyChange(scp);
+
+                if (RDR_Initialized)
+                    RDR_InvalidateObject(scp->fid.cell, scp->fid.volume, scp->fid.vnode, scp->fid.unique,
+                                         scp->fid.hash, scp->fileType, AFS_INVALIDATE_CALLBACK);
+
                 lock_ObtainWrite(&cm_scacheLock);
                 cm_ReleaseSCacheNoLock(scp);
                 lock_ObtainMutex(&cm_Freelance_Lock);
@@ -419,12 +480,17 @@ int cm_reInitLocalMountPoints() {
         }
     }
     lock_ReleaseWrite(&cm_scacheLock);
+    lock_ReleaseMutex(&cm_Freelance_Lock);
     osi_Log0(afsd_logp,"\tall old scp cleared!");
 
+    lock_ObtainWrite(&cm_data.rootSCachep->rw);
+    lock_ObtainMutex(&cm_Freelance_Lock);
     // we must free the memory that was allocated in the prev
     // cm_InitLocalMountPoints call
     osi_Log0(afsd_logp,"Removing old localmountpoints...  ");
     free(cm_localMountPoints);
+    cm_localMountPoints = NULL;
+    cm_noLocalMountPoints = 0;
     osi_Log0(afsd_logp,"\tall old localmountpoints cleared!");
 
     // now re-init the localmountpoints
@@ -439,18 +505,73 @@ int cm_reInitLocalMountPoints() {
 
     lock_ReleaseMutex(&cm_Freelance_Lock);
 
+    cm_GetCallback(cm_data.rootSCachep, cm_rootUserp, &req, 0);
+    lock_ReleaseWrite(&cm_data.rootSCachep->rw);
+
     osi_Log0(afsd_logp,"----- freelance reinit complete -----");
     return 0;
 }
 
+/*
+ * cm_enforceTrailingDot
+ *
+ * return 0 on failure, non-zero on success
+ *
+ */
+static int
+cm_enforceTrailingDot(char * line, size_t cchLine, DWORD *pdwSize)
+{
+    if (*pdwSize < 4) {
+        afsi_log("invalid string");
+        return 0;
+    }
 
-// yj: open up the registry and read all the local mount 
+    /* trailing white space first. */
+    if (line[(*pdwSize)-1] == '\0') {
+        while (isspace(line[(*pdwSize)-2])) {
+            line[(*pdwSize)-2] = '\0';
+            (*pdwSize)--;
+        }
+    } else {
+        while (isspace(line[(*pdwSize)-1])) {
+            line[(*pdwSize)-1] = '\0';
+            (*pdwSize)--;
+        }
+    }
+
+    /* then enforce the trailing dot requirement */
+    if (line[(*pdwSize)-1] == '\0' && line[(*pdwSize)-2] != '.') {
+        if ((*pdwSize) >= cchLine) {
+            afsi_log("no room for trailing dot");
+            return 0;
+        }
+        line[(*pdwSize)-1] = '.';
+        line[(*pdwSize)] = '\0';
+    } else if (line[(*pdwSize)-1] != '\0' && line[(*pdwSize)-1] != '.') {
+        if ((*pdwSize) >= cchLine) {
+            afsi_log("no room for trailing dot and nul");
+            return 0;
+        }
+        line[(*pdwSize)] = '.';
+        line[(*pdwSize)+1] = '\0';
+    } else if (line[(*pdwSize)-1] != '\0') {
+        if ((*pdwSize) >= cchLine) {
+            afsi_log("no room for trailing nul");
+            return 0;
+        }
+        line[(*pdwSize)] = '\0';
+    }
+    return 1;
+}
+
+
+// yj: open up the registry and read all the local mount
 // points that are stored there. Part of the initialization
 // process for the freelance client.
-/* to be called while holding freelance lock unless during init. */
+/* to be called while holding freelance lock. */
 long cm_InitLocalMountPoints() {
     FILE *fp;
-    int i;
+    unsigned int i;
     char line[512];
     char*t, *t2;
     cm_localMountPoint_t* aLocalMountPoint;
@@ -464,7 +585,7 @@ long cm_InitLocalMountPoints() {
     DWORD dwSymlinks = 0;
     FILETIME ftLastWriteTime;
 
-    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
+    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                       AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance",
                       0,
                       KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
@@ -484,20 +605,22 @@ long cm_InitLocalMountPoints() {
                          &ftLastWriteTime /* lpftLastWriteTime */
                          );
 
-        smb_UnixTimeFromLargeSearchTime(&FakeFreelanceModTime, &ftLastWriteTime);
+        cm_UnixTimeFromLargeSearchTime(&FakeFreelanceModTime, &ftLastWriteTime);
 
         if ( dwMountPoints == 0 ) {
             rootCellName[0] = '.';
             code = cm_GetRootCellName(&rootCellName[1]);
             if (code == 0) {
-                cm_FreelanceAddMount(&rootCellName[1], &rootCellName[1], "root.cell.", 0, NULL);
-                cm_FreelanceAddMount(rootCellName, &rootCellName[1], "root.cell.", 1, NULL);
-                cm_FreelanceAddMount(".root", &rootCellName[1], "root.afs.", 1, NULL);
+                lock_ReleaseMutex(&cm_Freelance_Lock);
+                cm_FreelanceAddMount(&rootCellName[1], &rootCellName[1], "root.cell", 0, NULL);
+                cm_FreelanceAddMount(rootCellName, &rootCellName[1], "root.cell", 1, NULL);
+                cm_FreelanceAddMount(".root", &rootCellName[1], "root.afs", 1, NULL);
+                lock_ObtainMutex(&cm_Freelance_Lock);
                 dwMountPoints = 3;
             }
         }
 
-        if (RegCreateKeyEx( HKEY_LOCAL_MACHINE, 
+        if (RegCreateKeyEx( HKEY_LOCAL_MACHINE,
                           AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance\\Symlinks",
                           0,
                           NULL,
@@ -533,7 +656,7 @@ long cm_InitLocalMountPoints() {
         // now we read n lines and parse them into local mount points
         // where n is the number of local mount points there are, as
         // determined above.
-        // Each line in the ini file represents 1 local mount point and 
+        // Each line in the ini file represents 1 local mount point and
         // is in the format xxx#yyy:zzz, where xxx is the directory
         // entry name, yyy is the cell name and zzz is the volume name.
         // #yyy:zzz together make up the mount point.
@@ -549,12 +672,13 @@ long cm_InitLocalMountPoints() {
                 continue;
             }
 
-            afsi_log("Mountpoint[%d] = %s",dwIndex, line);
+            /* make sure there is a trailing dot and a nul terminator */
+            if (!cm_enforceTrailingDot(line, sizeof(line), &dwSize)) {
+                cm_noLocalMountPoints--;
+                continue;
+            }
 
-            /* find the trailing dot; null terminate after it */
-            t2 = strrchr(line, '.');
-            if (t2)
-                *(t2+1) = '\0';
+            afsi_log("Mountpoint[%d] = %s", dwIndex, line);
 
             for ( t=line;*t;t++ ) {
                 if ( !isprint(*t) ) {
@@ -581,12 +705,12 @@ long cm_InitLocalMountPoints() {
             aLocalMountPoint->namep=malloc(t-line+1);
             strncpy(aLocalMountPoint->namep, line, t-line);
             aLocalMountPoint->namep[t-line] = '\0';
-		
+
             /* copy the mount point string */
             aLocalMountPoint->mountPointStringp=malloc(strlen(t));
             strncpy(aLocalMountPoint->mountPointStringp, t, strlen(t)-1);
             aLocalMountPoint->mountPointStringp[strlen(t)-1] = '\0';
-    
+
             osi_Log2(afsd_logp,"found mount point: name %s, string %s",
                       osi_LogSaveString(afsd_logp,aLocalMountPoint->namep),
                       osi_LogSaveString(afsd_logp,aLocalMountPoint->mountPointStringp));
@@ -606,12 +730,13 @@ long cm_InitLocalMountPoints() {
                 continue;
             }
 
-            afsi_log("Symlink[%d] = %s",dwIndex, line);
+            /* make sure there is a trailing dot and a nul terminator */
+            if (!cm_enforceTrailingDot(line, sizeof(line), &dwSize)) {
+                cm_noLocalMountPoints--;
+                continue;
+            }
 
-            /* find the trailing dot; null terminate after it */
-            t2 = strrchr(line, '.');
-            if (t2)
-                *(t2+1) = '\0';
+            afsi_log("Symlink[%d] = %s", dwIndex, line);
 
             for ( t=line;*t;t++ ) {
                 if ( !isprint(*t) ) {
@@ -637,12 +762,12 @@ long cm_InitLocalMountPoints() {
             aLocalMountPoint->namep=malloc(t-line+1);
             strncpy(aLocalMountPoint->namep, line, t-line);
             aLocalMountPoint->namep[t-line] = '\0';
-		
+
             /* copy the symlink string */
             aLocalMountPoint->mountPointStringp=malloc(strlen(t)-1);
             strncpy(aLocalMountPoint->mountPointStringp, t+1, strlen(t)-2);
             aLocalMountPoint->mountPointStringp[strlen(t)-2] = '\0';
-    
+
             osi_Log2(afsd_logp,"found symlink: name %s, string %s",
                       osi_LogSaveString(afsd_logp,aLocalMountPoint->namep),
                       osi_LogSaveString(afsd_logp,aLocalMountPoint->mountPointStringp));
@@ -656,7 +781,7 @@ long cm_InitLocalMountPoints() {
         return 0;
     }
 
-    /* What follows is the old code to read freelance mount points 
+    /* What follows is the old code to read freelance mount points
      * out of a text file modified to copy the data into the registry
      */
     cm_GetConfigDir(hdir, sizeof(hdir));
@@ -671,7 +796,7 @@ long cm_InitLocalMountPoints() {
         fp = fopen(hdir, "r");
     }
 
-    RegCreateKeyEx( HKEY_LOCAL_MACHINE, 
+    RegCreateKeyEx( HKEY_LOCAL_MACHINE,
                     AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance",
                     0,
                     NULL,
@@ -687,16 +812,18 @@ long cm_InitLocalMountPoints() {
         rootCellName[0] = '.';
       	code = cm_GetRootCellName(&rootCellName[1]);
         if (code == 0) {
-            cm_FreelanceAddMount(&rootCellName[1], &rootCellName[1], "root.cell.", 0, NULL);
-            cm_FreelanceAddMount(rootCellName, &rootCellName[1], "root.cell.", 1, NULL);
-            cm_FreelanceAddMount(".root", &rootCellName[1], "root.afs.", 1, NULL);
+            lock_ReleaseMutex(&cm_Freelance_Lock);
+            cm_FreelanceAddMount(&rootCellName[1], &rootCellName[1], "root.cell", 0, NULL);
+            cm_FreelanceAddMount(rootCellName, &rootCellName[1], "root.cell", 1, NULL);
+            cm_FreelanceAddMount(".root", &rootCellName[1], "root.afs", 1, NULL);
+            lock_ObtainMutex(&cm_Freelance_Lock);
         }
         return 0;
     }
 
     // we successfully opened the file
     osi_Log0(afsd_logp,"opened afs_freelance.ini");
-	
+
     // now we read the first line to see how many entries
     // there are
     fgets(line, sizeof(line), fp);
@@ -722,7 +849,7 @@ long cm_InitLocalMountPoints() {
     // now we read n lines and parse them into local mount points
     // where n is the number of local mount points there are, as
     // determined above.
-    // Each line in the ini file represents 1 local mount point and 
+    // Each line in the ini file represents 1 local mount point and
     // is in the format xxx#yyy:zzz, where xxx is the directory
     // entry name, yyy is the cell name and zzz is the volume name.
     // #yyy:zzz together make up the mount point.
@@ -795,14 +922,14 @@ long cm_FreelanceMountPointExists(char * filename, int prefix_ok)
     DWORD dwType, dwSize;
     DWORD dwMountPoints;
     DWORD dwIndex;
-        
+
     lock_ObtainMutex(&cm_Freelance_Lock);
 
-    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
+    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                       AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance",
                       0,
                       KEY_READ|KEY_QUERY_VALUE,
-                      &hkFreelance) == ERROR_SUCCESS) 
+                      &hkFreelance) == ERROR_SUCCESS)
     {
         RegQueryInfoKey( hkFreelance,
                          NULL,  /* lpClass */
@@ -877,14 +1004,14 @@ long cm_FreelanceSymlinkExists(char * filename, int prefix_ok)
     DWORD dwType, dwSize;
     DWORD dwSymlinks;
     DWORD dwIndex;
-        
+
     lock_ObtainMutex(&cm_Freelance_Lock);
 
-    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
+    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                       AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance\\Symlinks",
                       0,
                       KEY_READ|KEY_QUERY_VALUE,
-                      &hkFreelance) == ERROR_SUCCESS) 
+                      &hkFreelance) == ERROR_SUCCESS)
     {
         RegQueryInfoKey( hkFreelance,
                          NULL,  /* lpClass */
@@ -957,38 +1084,41 @@ long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw,
     DWORD dwType, dwSize;
     DWORD dwMountPoints;
     DWORD dwIndex;
+    afs_uint32 code = 0;
 
     /* before adding, verify the cell name; if it is not a valid cell,
        don't add the mount point.
        allow partial matches as a means of poor man's alias. */
     /* major performance issue? */
     osi_Log4(afsd_logp,"Freelance Add Mount request: filename=%s cellname=%s volume=%s %s",
-              osi_LogSaveString(afsd_logp,filename), 
-              osi_LogSaveString(afsd_logp,cellname), 
-              osi_LogSaveString(afsd_logp,volume), 
+              osi_LogSaveString(afsd_logp,filename),
+              osi_LogSaveString(afsd_logp,cellname),
+              osi_LogSaveString(afsd_logp,volume),
               rw ? "rw" : "ro");
 
     if ( filename[0] == '\0' || cellname[0] == '\0' || volume[0] == '\0' )
-        return -1;
+        return CM_ERROR_INVAL;
+
+    if ( cm_FreelanceMountPointExists(filename, 0) ||
+         cm_FreelanceSymlinkExists(filename, 0) ) {
+        code = CM_ERROR_EXISTS;
+        goto done;
+    }
 
     if (cellname[0] == '.') {
         if (!cm_GetCell_Gen(&cellname[1], fullname, CM_FLAG_CREATE))
-            return -1;
+            return CM_ERROR_INVAL;
     } else {
         if (!cm_GetCell_Gen(cellname, fullname, CM_FLAG_CREATE))
-            return -1;
+            return CM_ERROR_INVAL;
     }
 
-    if ( cm_FreelanceMountPointExists(filename, 0) ||
-         cm_FreelanceSymlinkExists(filename, 0) )
-        return -1;
-    
-    osi_Log1(afsd_logp,"Freelance Adding Mount for Cell: %s", 
+    osi_Log1(afsd_logp,"Freelance Adding Mount for Cell: %s",
               osi_LogSaveString(afsd_logp,cellname));
 
     lock_ObtainMutex(&cm_Freelance_Lock);
 
-    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
+    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                       AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance",
                       0,
                       KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
@@ -1009,12 +1139,12 @@ long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw,
                          );
 
         if (rw)
-            sprintf(line, "%s%%%s:%s", filename, fullname, volume);
+            sprintf(line, "%s%%%s:%s.", filename, fullname, volume);
         else
-            sprintf(line, "%s#%s:%s", filename, fullname, volume);
+            sprintf(line, "%s#%s:%s.", filename, fullname, volume);
 
         /* If we are adding a new value, there must be an unused name
-         * within the range 0 to dwMountPoints 
+         * within the range 0 to dwMountPoints
          */
         for ( dwIndex = 0; dwIndex <= dwMountPoints; dwIndex++ ) {
             char szIndex[16];
@@ -1030,7 +1160,7 @@ long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw,
                 break;
             } else {
                 int len = (int)strlen(filename);
-                if ( dwType == REG_SZ && !strncmp(filename, szMount, len) && 
+                if ( dwType == REG_SZ && !strncmp(filename, szMount, len) &&
                      (szMount[len] == '%' || szMount[len] == '#')) {
                     /* Replace the existing value */
                     dwType = REG_SZ;
@@ -1041,7 +1171,7 @@ long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw,
             }
         }
         RegCloseKey(hkFreelance);
-    } else 
+    } else
     {
         cm_GetConfigDir(hfile, sizeof(hfile));
         strcat(hfile, AFS_FREELANCE_INI);
@@ -1055,18 +1185,44 @@ long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw,
         fprintf(fp, "%d", n);
         fseek(fp, 0, SEEK_END);
         if (rw)
-            fprintf(fp, "%s%%%s:%s\n", filename, fullname, volume);
+            fprintf(fp, "%s%%%s:%s.\n", filename, fullname, volume);
         else
-            fprintf(fp, "%s#%s:%s\n", filename, fullname, volume);
+            fprintf(fp, "%s#%s:%s.\n", filename, fullname, volume);
         fclose(fp);
     }
+
+    /* Do this while we are holding the lock */
+    cm_noteLocalMountPointChange(TRUE);
     lock_ReleaseMutex(&cm_Freelance_Lock);
 
-    /* cm_reInitLocalMountPoints(); */
-    if (fidp)
-        cm_SetFid(fidp, fidp->cell, fidp->volume, ++cm_noLocalMountPoints, 1);
-    cm_noteLocalMountPointChange();
-    return 0;
+  done:
+    if (fidp) {
+        cm_req_t req;
+        cm_scache_t *scp;
+        clientchar_t *cpath;
+
+        cm_InitReq(&req);
+
+        cpath = cm_FsStringToClientStringAlloc(filename, -1, NULL);
+        if (!cpath)
+            return CM_ERROR_NOSUCHPATH;
+
+        if (cm_getLocalMountPointChange()) {	// check for changes
+            cm_clearLocalMountPointChange();    // clear the changefile
+            cm_reInitLocalMountPoints();	// start reinit
+        }
+
+        code = cm_NameI(cm_RootSCachep(cm_rootUserp, &req), cpath,
+                        CM_FLAG_FOLLOW | CM_FLAG_CASEFOLD | CM_FLAG_DFS_REFERRAL,
+                        cm_rootUserp, NULL, &req, &scp);
+        free(cpath);
+        if (code)
+            return code;
+        *fidp = scp->fid;
+        cm_ReleaseSCache(scp);
+    }
+
+    return code;
 }
 
 long cm_FreelanceRemoveMount(char *toremove)
@@ -1085,7 +1241,7 @@ long cm_FreelanceRemoveMount(char *toremove)
 
     lock_ObtainMutex(&cm_Freelance_Lock);
 
-    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
+    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                       AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance",
                       0,
                       KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
@@ -1125,7 +1281,7 @@ long cm_FreelanceRemoveMount(char *toremove)
             }
         }
         RegCloseKey(hkFreelance);
-    } else 
+    } else
     {
         cm_GetConfigDir(hfile, sizeof(hfile));
         strcat(hfile, AFS_FREELANCE_INI);
@@ -1167,53 +1323,64 @@ long cm_FreelanceRemoveMount(char *toremove)
             rename(hfile2, hfile);
         }
     }
-    
-    lock_ReleaseMutex(&cm_Freelance_Lock);
+
     if (found) {
-        cm_noteLocalMountPointChange();
-        return 0;
-    } else
-        return CM_ERROR_NOSUCHFILE;
+        /* Do this while we are holding the lock */
+        cm_noteLocalMountPointChange(TRUE);
+    }
+    lock_ReleaseMutex(&cm_Freelance_Lock);
+    return (found ? 0 : CM_ERROR_NOSUCHFILE);
 }
 
 long cm_FreelanceAddSymlink(char *filename, char *destination, cm_fid_t *fidp)
 {
     char line[512];
-    char fullname[CELL_MAXNAMELEN];
+    char fullname[CELL_MAXNAMELEN] = "";
     int alias = 0;
     HKEY hkFreelanceSymlinks = 0;
     DWORD dwType, dwSize;
     DWORD dwSymlinks;
     DWORD dwIndex;
+    afs_uint32 code = 0;
 
-    /* before adding, verify the filename.  If it is already in use, either as 
+    /*
+     * before adding, verify the filename.  If it is already in use, either as
      * as mount point or a cellname, do not permit the creation of the symlink.
      */
     osi_Log2(afsd_logp,"Freelance Add Symlink request: filename=%s destination=%s",
-              osi_LogSaveString(afsd_logp,filename), 
+              osi_LogSaveString(afsd_logp,filename),
               osi_LogSaveString(afsd_logp,destination));
-    
+
     if ( filename[0] == '\0' || destination[0] == '\0' )
         return CM_ERROR_INVAL;
 
-    fullname[0] = '\0';
-    if (filename[0] == '.') {
-        cm_GetCell_Gen(&filename[1], fullname, CM_FLAG_CREATE);
-        if (cm_stricmp_utf8(&filename[1],fullname) == 0)
-            return CM_ERROR_EXISTS;
-    } else {
-        cm_GetCell_Gen(filename, fullname, CM_FLAG_CREATE);
-        if (cm_stricmp_utf8(filename,fullname) == 0)
-            return CM_ERROR_EXISTS;
-    }
+    /* Do not create the symlink if the name ends in a dot */
+    if ( filename[strlen(filename)-1] == '.')
+        return CM_ERROR_INVAL;
 
     if ( cm_FreelanceMountPointExists(filename, 0) ||
-         cm_FreelanceSymlinkExists(filename, 0) )
-        return CM_ERROR_EXISTS;
+         cm_FreelanceSymlinkExists(filename, 0) ) {
+        code = CM_ERROR_EXISTS;
+        goto done;
+    }
+
+    if (filename[0] == '.') {
+        cm_GetCell_Gen(&filename[1], fullname, CM_FLAG_CREATE);
+        if (cm_stricmp_utf8(&filename[1],fullname) == 0) {
+            code = CM_ERROR_EXISTS;
+            goto done;
+        }
+    } else {
+        cm_GetCell_Gen(filename, fullname, CM_FLAG_CREATE);
+        if (cm_stricmp_utf8(filename,fullname) == 0) {
+            code = CM_ERROR_EXISTS;
+            goto done;
+        }
+    }
 
     lock_ObtainMutex(&cm_Freelance_Lock);
 
-    if (RegCreateKeyEx( HKEY_LOCAL_MACHINE, 
+    if (RegCreateKeyEx( HKEY_LOCAL_MACHINE,
                         AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance\\Symlinks",
                         0,
                         NULL,
@@ -1240,7 +1407,7 @@ long cm_FreelanceAddSymlink(char *filename, char *destination, cm_fid_t *fidp)
         sprintf(line, "%s:%s.", filename, destination);
 
         /* If we are adding a new value, there must be an unused name
-         * within the range 0 to dwSymlinks 
+         * within the range 0 to dwSymlinks
          */
         for ( dwIndex = 0; dwIndex <= dwSymlinks; dwIndex++ ) {
             char szIndex[16];
@@ -1266,14 +1433,41 @@ long cm_FreelanceAddSymlink(char *filename, char *destination, cm_fid_t *fidp)
             }
         }
         RegCloseKey(hkFreelanceSymlinks);
-    } 
+    }
+
+    /* Do this while we are holding the lock */
+    cm_noteLocalMountPointChange(TRUE);
     lock_ReleaseMutex(&cm_Freelance_Lock);
 
-    /* cm_reInitLocalMountPoints(); */
-    if (fidp)
-        cm_SetFid(fidp, fidp->cell, fidp->volume, ++cm_noLocalMountPoints, 1);
-    cm_noteLocalMountPointChange();
-    return 0;
+  done:
+    if (fidp) {
+        cm_req_t req;
+        cm_scache_t *scp;
+        clientchar_t *cpath;
+
+        cm_InitReq(&req);
+
+        cpath = cm_FsStringToClientStringAlloc(filename, -1, NULL);
+        if (!cpath) {
+            code = CM_ERROR_NOSUCHPATH;
+        } else {
+            if (cm_getLocalMountPointChange()) {	// check for changes
+                cm_clearLocalMountPointChange();    // clear the changefile
+                cm_reInitLocalMountPoints();	// start reinit
+            }
+
+            code = cm_NameI(cm_RootSCachep(cm_rootUserp, &req), cpath,
+                             CM_FLAG_FOLLOW | CM_FLAG_CASEFOLD | CM_FLAG_DFS_REFERRAL,
+                             cm_rootUserp, NULL, &req, &scp);
+            free(cpath);
+            if (code == 0) {
+                *fidp = scp->fid;
+                cm_ReleaseSCache(scp);
+            }
+        }
+    }
+
+    return code;
 }
 
 long cm_FreelanceRemoveSymlink(char *toremove)
@@ -1289,7 +1483,7 @@ long cm_FreelanceRemoveSymlink(char *toremove)
 
     lock_ObtainMutex(&cm_Freelance_Lock);
 
-    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
+    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                       AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance\\Symlinks",
                       0,
                       KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
@@ -1328,12 +1522,53 @@ long cm_FreelanceRemoveSymlink(char *toremove)
         }
         RegCloseKey(hkFreelanceSymlinks);
     }
-    
-    lock_ReleaseMutex(&cm_Freelance_Lock);
+
     if (found) {
-        cm_noteLocalMountPointChange();
-        return 0;
-    } else
-        return CM_ERROR_NOSUCHFILE;
+        /* Do this while we are holding the lock */
+        cm_noteLocalMountPointChange(TRUE);
+    }
+    lock_ReleaseMutex(&cm_Freelance_Lock);
+    return (found ? 0 : CM_ERROR_NOSUCHFILE);
+}
+
+long
+cm_FreelanceFetchMountPointString(cm_scache_t *scp)
+{
+    lock_ObtainMutex(&cm_Freelance_Lock);
+    if (!scp->mountPointStringp[0] &&
+        scp->fid.cell == AFS_FAKE_ROOT_CELL_ID &&
+        scp->fid.volume == AFS_FAKE_ROOT_VOL_ID &&
+        (afs_int32)(scp->fid.unique - cm_data.fakeUnique) - 1 >= 0 &&
+        scp->fid.unique - cm_data.fakeUnique <= cm_noLocalMountPoints) {
+        strncpy(scp->mountPointStringp, cm_localMountPoints[scp->fid.unique-cm_data.fakeUnique-1].mountPointStringp, MOUNTPOINTLEN);
+        scp->mountPointStringp[MOUNTPOINTLEN-1] = 0;	/* null terminate */
+    }
+    lock_ReleaseMutex(&cm_Freelance_Lock);
+
+    return 0;
+}
+
+long
+cm_FreelanceFetchFileType(cm_scache_t *scp)
+{
+    lock_ObtainMutex(&cm_Freelance_Lock);
+    if (scp->fid.cell == AFS_FAKE_ROOT_CELL_ID &&
+        scp->fid.volume == AFS_FAKE_ROOT_VOL_ID &&
+        (afs_int32)(scp->fid.unique - cm_data.fakeUnique) - 1 >= 0 &&
+        scp->fid.unique - cm_data.fakeUnique <= cm_noLocalMountPoints)
+    {
+        scp->fileType = cm_localMountPoints[scp->fid.unique-cm_data.fakeUnique-1].fileType;
+
+        if (scp->fileType == CM_SCACHETYPE_SYMLINK &&
+            !strnicmp(cm_localMountPoints[scp->fid.unique-cm_data.fakeUnique-1].mountPointStringp, "msdfs:", strlen("msdfs:")) )
+        {
+            scp->fileType = CM_SCACHETYPE_DFSLINK;
+        }
+    } else {
+        scp->fileType = CM_SCACHETYPE_INVALID;
+    }
+    lock_ReleaseMutex(&cm_Freelance_Lock);
+
+    return 0;
 }
 #endif /* AFS_FREELANCE_CLIENT */
