@@ -1,3 +1,8 @@
+
+#include <afsconfig.h>
+#include <afs/param.h>
+#include <roken.h>
+
 #include <windows.h>
 #include <tchar.h>
 #include "afsd.h"
@@ -6,6 +11,9 @@
 extern void afsi_log(char *pattern, ...);
 extern DWORD cm_ValidateCache;
 
+static HANDLE hMemoryMappedFile = NULL;
+static HANDLE hCacheHeap = NULL;
+
 afs_uint64
 GranularityAdjustment(afs_uint64 size)
 {
@@ -22,7 +30,7 @@ GranularityAdjustment(afs_uint64 size)
     return size;
 }
 
-afs_uint64 
+afs_uint64
 ComputeSizeOfConfigData(void)
 {
     afs_uint64 size;
@@ -62,7 +70,7 @@ ComputeSizeOfCells(DWORD maxcells)
     return size;
 }
 
-afs_uint64 
+afs_uint64
 ComputeSizeOfACLCache(DWORD stats)
 {
     afs_uint64 size;
@@ -70,7 +78,7 @@ ComputeSizeOfACLCache(DWORD stats)
     return size;
 }
 
-afs_uint64 
+afs_uint64
 ComputeSizeOfSCache(DWORD stats)
 {
     afs_uint64 size;
@@ -78,7 +86,7 @@ ComputeSizeOfSCache(DWORD stats)
     return size;
 }
 
-afs_uint64 
+afs_uint64
 ComputeSizeOfSCacheHT(DWORD stats)
 {
     afs_uint64 size;
@@ -86,7 +94,7 @@ ComputeSizeOfSCacheHT(DWORD stats)
     return size;
 }
 
-afs_uint64 
+afs_uint64
 ComputeSizeOfDNLCache(void)
 {
     afs_uint64 size;
@@ -94,7 +102,7 @@ ComputeSizeOfDNLCache(void)
     return size;
 }
 
-afs_uint64 
+afs_uint64
 ComputeSizeOfDataBuffers(afs_uint64 cacheBlocks, DWORD blockSize)
 {
     afs_uint64 size;
@@ -102,7 +110,7 @@ ComputeSizeOfDataBuffers(afs_uint64 cacheBlocks, DWORD blockSize)
     return size;
 }
 
-afs_uint64 
+afs_uint64
 ComputeSizeOfDataHT(afs_uint64 cacheBlocks)
 {
     afs_uint64 size;
@@ -110,7 +118,7 @@ ComputeSizeOfDataHT(afs_uint64 cacheBlocks)
     return size;
 }
 
-afs_uint64 
+afs_uint64
 ComputeSizeOfDataHeaders(afs_uint64 cacheBlocks)
 {
     afs_uint64 size;
@@ -122,20 +130,20 @@ afs_uint64
 ComputeSizeOfMappingFile(DWORD stats, DWORD maxVols, DWORD maxCells, DWORD chunkSize, afs_uint64 cacheBlocks, DWORD blockSize)
 {
     afs_uint64 size;
-    
+
     size       =  ComputeSizeOfConfigData()
-               +  ComputeSizeOfVolumes(maxVols) 
-               +  4 * ComputeSizeOfVolumeHT(maxVols) 
-               +  ComputeSizeOfCells(maxCells) 
+               +  ComputeSizeOfVolumes(maxVols)
+               +  4 * ComputeSizeOfVolumeHT(maxVols)
+               +  ComputeSizeOfCells(maxCells)
                +  2 * ComputeSizeOfCellHT(maxCells)
                +  ComputeSizeOfACLCache(stats)
                +  ComputeSizeOfSCache(stats)
                +  ComputeSizeOfSCacheHT(stats)
                +  ComputeSizeOfDNLCache()
-               +  ComputeSizeOfDataBuffers(cacheBlocks, blockSize) 
-               +  2 * ComputeSizeOfDataHT(cacheBlocks) 
+               +  ComputeSizeOfDataBuffers(cacheBlocks, blockSize)
+               +  2 * ComputeSizeOfDataHT(cacheBlocks)
                +  ComputeSizeOfDataHeaders(cacheBlocks);
-    return size;    
+    return size;
 }
 
 /* Create a security attribute structure suitable for use when the cache file
@@ -182,7 +190,7 @@ PSECURITY_ATTRIBUTES CreateCacheFileSA()
     psa->bInheritHandle = FALSE;
 
     return psa;
-}       
+}
 
 
 /* Free a security attribute structure created by CreateCacheFileSA() */
@@ -195,9 +203,7 @@ VOID FreeCacheFileSA(PSECURITY_ATTRIBUTES psa)
     GlobalFree(pAcl);
     GlobalFree(psa->lpSecurityDescriptor);
     GlobalFree(psa);
-}       
-
-static HANDLE hMemoryMappedFile = NULL;
+}
 
 int
 cm_IsCacheValid(void)
@@ -255,16 +261,21 @@ cm_ShutdownMappedMemory(void)
     cm_ShutdownCell();
     cm_ShutdownVolume();
 
-    if (cm_ValidateCache == 2)
-        dirty = !cm_IsCacheValid();
+    if (hCacheHeap) {
+        HeapFree(hCacheHeap, 0, cm_data.baseAddress);
+        HeapDestroy(hCacheHeap);
+        afsi_log("Memory Heap has been destroyed");
+    } else {
+        if (cm_ValidateCache == 2)
+            dirty = !cm_IsCacheValid();
 
-    *config_data_p = cm_data;
-    config_data_p->dirty = dirty;
-    UnmapViewOfFile(config_data_p);
-    CloseHandle(hMemoryMappedFile);
-    hMemoryMappedFile = NULL;
-
-    afsi_log("Memory Mapped File has been closed");
+        *config_data_p = cm_data;
+        config_data_p->dirty = dirty;
+        UnmapViewOfFile(config_data_p);
+        CloseHandle(hMemoryMappedFile);
+        hMemoryMappedFile = NULL;
+        afsi_log("Memory Mapped File has been closed");
+    }
     return 0;
 }
 
@@ -278,20 +289,20 @@ cm_ValidateMappedMemory(char * cachePath)
     afs_uint64 mappingSize;
     char * baseAddress = NULL;
     cm_config_data_t * config_data_p;
-        
+
     psa = CreateCacheFileSA();
     hf = CreateFile( cachePath,
                      GENERIC_READ | GENERIC_WRITE,
                      FILE_SHARE_READ | FILE_SHARE_WRITE,
                      psa,
                      OPEN_EXISTING,
-                     FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM | 
+                     FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM |
                      FILE_ATTRIBUTE_NOT_CONTENT_INDEXED | FILE_FLAG_RANDOM_ACCESS,
                      NULL);
     FreeCacheFileSA(psa);
 
     if (hf == INVALID_HANDLE_VALUE) {
-        fprintf(stderr, "Error creating cache file \"%s\" error %d\n", 
+        fprintf(stderr, "Error creating cache file \"%s\" error %d\n",
                  cachePath, GetLastError());
         return CM_ERROR_INVAL;
     }
@@ -306,11 +317,11 @@ cm_ValidateMappedMemory(char * cachePath)
     afsi_log("Existing File Size: %08X:%08X",
               fileInfo.nFileSizeHigh,
               fileInfo.nFileSizeLow);
-            
+
     hm = CreateFileMapping( hf,
                             NULL,
                             PAGE_READWRITE,
-                            0, 
+                            0,
                             sizeof(cm_config_data_t),
                             NULL);
     if (hm == NULL) {
@@ -321,7 +332,7 @@ cm_ValidateMappedMemory(char * cachePath)
             hm = CreateFileMapping( hf,
                                     NULL,
                                     PAGE_READWRITE,
-                                    0, 
+                                    0,
                                     fileInfo.nFileSizeLow,
                                     NULL);
             if (hm == NULL) {
@@ -335,7 +346,7 @@ cm_ValidateMappedMemory(char * cachePath)
                     return CM_ERROR_INVAL;
                 }
             } else {
-                fprintf(stderr, "Retry with file size (%lX) succeeds", 
+                fprintf(stderr, "Retry with file size (%lX) succeeds",
                          fileInfo.nFileSizeLow);
             }
         } else {
@@ -348,7 +359,7 @@ cm_ValidateMappedMemory(char * cachePath)
 
     config_data_p = MapViewOfFile( hm,
                                    FILE_MAP_READ,
-                                   0, 0,   
+                                   0, 0,
                                    sizeof(cm_config_data_t));
     if ( config_data_p == NULL ) {
         fprintf(stderr, "Unable to MapViewOfFile\n");
@@ -361,7 +372,7 @@ cm_ValidateMappedMemory(char * cachePath)
     if ( config_data_p->dirty ) {
         fprintf(stderr, "Previous session terminated prematurely\n");
         UnmapViewOfFile(config_data_p);
-        CloseHandle(hm);               
+        CloseHandle(hm);
         CloseHandle(hf);
         return CM_ERROR_INVAL;
     }
@@ -374,7 +385,7 @@ cm_ValidateMappedMemory(char * cachePath)
     hm = CreateFileMapping( hf,
                             NULL,
                             PAGE_READWRITE,
-			    (DWORD)(mappingSize >> 32), 
+			    (DWORD)(mappingSize >> 32),
 			    (DWORD)(mappingSize & 0xFFFFFFFF),
                             NULL);
     if (hm == NULL) {
@@ -389,7 +400,7 @@ cm_ValidateMappedMemory(char * cachePath)
         CloseHandle(hf);
         return CM_ERROR_INVAL;
     }
-    
+
     baseAddress = MapViewOfFileEx( hm,
                                    FILE_MAP_ALL_ACCESS,
                                    0, 0,
@@ -424,16 +435,16 @@ cm_ValidateMappedMemory(char * cachePath)
     fprintf(stderr,"  blockSize      = %u\n", config_data_p->blockSize);
     fprintf(stderr,"  bufferSize     = %I64u\n", config_data_p->bufferSize);
     fprintf(stderr,"  cacheType      = %u\n", config_data_p->cacheType);
-    fprintf(stderr,"  volumeHashTableSize = %u", config_data_p->volumeHashTableSize);
+    fprintf(stderr,"  volumeHashTableSize = %u\n", config_data_p->volumeHashTableSize);
     fprintf(stderr,"  currentVolumes = %u\n", config_data_p->currentVolumes);
     fprintf(stderr,"  maxVolumes     = %u\n", config_data_p->maxVolumes);
-    fprintf(stderr,"  cellHashTableSize = %u", config_data_p->cellHashTableSize);
+    fprintf(stderr,"  cellHashTableSize = %u\n", config_data_p->cellHashTableSize);
     fprintf(stderr,"  currentCells   = %u\n", config_data_p->currentCells);
     fprintf(stderr,"  maxCells       = %u\n", config_data_p->maxCells);
     fprintf(stderr,"  scacheHashTableSize  = %u\n", config_data_p->scacheHashTableSize);
     fprintf(stderr,"  currentSCaches = %u\n", config_data_p->currentSCaches);
     fprintf(stderr,"  maxSCaches     = %u\n", config_data_p->maxSCaches);
-    cm_data = *config_data_p;      
+    cm_data = *config_data_p;
 
     // perform validation of persisted data structures
     // if there is a failure, start from scratch
@@ -448,7 +459,7 @@ cm_ValidateMappedMemory(char * cachePath)
     return 0;
 }
 
-DWORD 
+DWORD
 GetVolSerialNumber(char * cachePath)
 {
     char rootpath[128];
@@ -456,7 +467,7 @@ GetVolSerialNumber(char * cachePath)
     DWORD serial = 0;
 
     if ( cachePath[0] == '\\' && cachePath[1] == '\\' ||
-	 cachePath[0] == '/' && cachePath[1] == '/' ) 
+	 cachePath[0] == '/' && cachePath[1] == '/' )
     {
 	rootpath[0]=rootpath[1]='\\';
 	for ( i=2; cachePath[i]; i++ ) {
@@ -494,7 +505,7 @@ BOOL GetTextualSid( PSID pSid, PBYTE TextualSid, LPDWORD lpdwBufferLen )
     DWORD dwSidSize;
 
     // Validate the binary SID.
-    if(!IsValidSid(pSid)) 
+    if(!IsValidSid(pSid))
 	return FALSE;
 
     // Get the identifier authority value from the SID.
@@ -556,17 +567,17 @@ BOOL GetTextualSid( PSID pSid, PBYTE TextualSid, LPDWORD lpdwBufferLen )
     return TRUE;
 }
 
-PBYTE 
+PBYTE
 IsSubAuthValid( PBYTE SidData, DWORD SidLength )
 {
     PBYTE	sidPtr;
 
     sidPtr = NULL;
     if ( SidLength % sizeof(DWORD) == 0 )  {
-	for ( sidPtr = SidData + SidLength - 5*sizeof(DWORD); 
-	      sidPtr >= SidData; 
+	for ( sidPtr = SidData + SidLength - 5*sizeof(DWORD);
+	      sidPtr >= SidData;
 	      sidPtr -= sizeof(DWORD) )
-	    if ( ((PDWORD)sidPtr)[1] == 0x05000000 &&  
+	    if ( ((PDWORD)sidPtr)[1] == 0x05000000 &&
 		 ((PDWORD)sidPtr)[2] == 0x00000015 )
 		break;
 	if ( sidPtr < SidData )
@@ -591,7 +602,7 @@ GetMachineSid(PBYTE SidBuffer, DWORD SidSize)
     //
     // Read the last subauthority of the current computer SID
     //
-    if( RegOpenKey( HKEY_LOCAL_MACHINE, "SECURITY\\SAM\\Domains\\Account", 
+    if( RegOpenKey( HKEY_LOCAL_MACHINE, "SECURITY\\SAM\\Domains\\Account",
 		    &hKey) != ERROR_SUCCESS ) {
 	return FALSE;
     }
@@ -622,15 +633,11 @@ GetMachineSid(PBYTE SidBuffer, DWORD SidSize)
 }
 
 int
-cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chunkSize, 
-                    afs_uint64 cacheBlocks, afs_uint32 blockSize)
+cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD maxVols, DWORD maxCells,
+                    DWORD chunkSize, afs_uint64 cacheBlocks, afs_uint32 blockSize)
 {
-    HANDLE hf = INVALID_HANDLE_VALUE, hm;
-    PSECURITY_ATTRIBUTES psa;
-    int newFile = 1;
     afs_uint64 mappingSize;
-    DWORD maxVols = stats/2;
-    DWORD maxCells = stats/4;
+    int newCache = 1;
     DWORD volumeSerialNumber = 0;
     DWORD sidStringSize = 0;
     DWORD rc;
@@ -644,24 +651,38 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
 
     mappingSize = ComputeSizeOfMappingFile(stats, maxVols, maxCells, chunkSize, cacheBlocks, blockSize);
 
-    if ( !virtualCache ) {
+    if ( virtualCache ) {
+        hCacheHeap = HeapCreate( HEAP_GENERATE_EXCEPTIONS, 0, 0);
+
+        baseAddress = HeapAlloc(hCacheHeap, 0, mappingSize);
+
+        if (baseAddress == NULL) {
+            afsi_log("Error allocating Virtual Memory gle=%d",
+                     GetLastError());
+            return CM_ERROR_INVAL;
+        }
+        newCache = 1;
+    } else {
+        HANDLE hf = INVALID_HANDLE_VALUE, hm;
+        PSECURITY_ATTRIBUTES psa;
+
         psa = CreateCacheFileSA();
         hf = CreateFile( cachePath,
                          GENERIC_READ | GENERIC_WRITE,
                          FILE_SHARE_READ | FILE_SHARE_WRITE,
                          psa,
                          OPEN_ALWAYS,
-                         FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM | 
+                         FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM |
                          FILE_ATTRIBUTE_NOT_CONTENT_INDEXED | FILE_FLAG_RANDOM_ACCESS,
                          NULL);
         FreeCacheFileSA(psa);
 
         if (hf == INVALID_HANDLE_VALUE) {
-            afsi_log("Error creating cache file \"%s\" error %d", 
+            afsi_log("Error creating cache file \"%s\" error %d",
                       cachePath, GetLastError());
             return CM_ERROR_INVAL;
         }
-        
+
         if ( GetLastError() == ERROR_ALREADY_EXISTS ) {
             BY_HANDLE_FILE_INFORMATION fileInfo;
 
@@ -683,13 +704,13 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
                                      FILE_SHARE_READ | FILE_SHARE_WRITE,
                                      psa,
                                      TRUNCATE_EXISTING,
-                                     FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM | 
+                                     FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM |
                                      FILE_ATTRIBUTE_NOT_CONTENT_INDEXED | FILE_FLAG_RANDOM_ACCESS,
                                      NULL);
                     FreeCacheFileSA(psa);
 
                     if (hf == INVALID_HANDLE_VALUE) {
-                        afsi_log("Error creating cache file \"%s\" error %d", 
+                        afsi_log("Error creating cache file \"%s\" error %d",
                                   cachePath, GetLastError());
                         return CM_ERROR_INVAL;
                     }
@@ -704,7 +725,7 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
             hm = CreateFileMapping( hf,
                                     NULL,
                                     PAGE_READWRITE,
-                                    0, 
+                                    0,
                                     sizeof(cm_config_data_t),
                                     NULL);
             if (hm == NULL) {
@@ -715,7 +736,7 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
                     hm = CreateFileMapping( hf,
                                             NULL,
                                             PAGE_READWRITE,
-                                            (DWORD)(mappingSize >> 32), 
+                                            (DWORD)(mappingSize >> 32),
                                             (DWORD)(mappingSize & 0xFFFFFFFF),
                                             NULL);
                     if (hm == NULL) {
@@ -741,7 +762,7 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
 
             config_data_p = MapViewOfFile( hm,
                                            FILE_MAP_READ,
-                                           0, 0,   
+                                           0, 0,
                                            sizeof(cm_config_data_t));
             if ( config_data_p == NULL ) {
                 if (hf != INVALID_HANDLE_VALUE)
@@ -764,7 +785,7 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
                     afsi_log("Previous session terminated prematurely");
                 } else {
                     baseAddress = config_data_p->baseAddress;
-                    newFile = 0;
+                    newCache = 0;
                 }
             } else {
                 afsi_log("Configuration changed or Not a persistent cache file");
@@ -772,54 +793,56 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
             UnmapViewOfFile(config_data_p);
             CloseHandle(hm);
         }
-    }
 
-    hm = CreateFileMapping( hf,
-                            NULL,
-                            PAGE_READWRITE,
-			    (DWORD)(mappingSize >> 32), 
-			    (DWORD)(mappingSize & 0xFFFFFFFF),
-                            NULL);
-    if (hm == NULL) {
-        if (GetLastError() == ERROR_DISK_FULL) {
-            afsi_log("Error creating file mapping for \"%s\": disk full [2]",
-                      cachePath);
-            return CM_ERROR_TOOMANYBUFS;
-        }
-        afsi_log("Error creating file mapping for \"%s\": %d",
-                  cachePath, GetLastError());
-        return CM_ERROR_INVAL;
-    }
-    baseAddress = MapViewOfFileEx( hm,
-                                   FILE_MAP_ALL_ACCESS,
-                                   0, 
-				   0, 
-				   (SIZE_T)mappingSize,
-                                   baseAddress );
-    if (baseAddress == NULL) {
-        afsi_log("Error mapping view of file: %d", GetLastError());
-        baseAddress = MapViewOfFile( hm,
-                                     FILE_MAP_ALL_ACCESS,
-                                     0, 
-				     0, 
-				     (SIZE_T)mappingSize);
-        if (baseAddress == NULL) {
-            if (hf != INVALID_HANDLE_VALUE)
-                CloseHandle(hf);
-            CloseHandle(hm);
+        hm = CreateFileMapping( hf,
+                                NULL,
+                                PAGE_READWRITE,
+                                (DWORD)(mappingSize >> 32),
+                                (DWORD)(mappingSize & 0xFFFFFFFF),
+                                NULL);
+        if (hm == NULL) {
+            if (GetLastError() == ERROR_DISK_FULL) {
+                afsi_log("Error creating file mapping for \"%s\": disk full [2]",
+                          cachePath);
+                return CM_ERROR_TOOMANYBUFS;
+            }
+            afsi_log("Error creating file mapping for \"%s\": %d",
+                      cachePath, GetLastError());
             return CM_ERROR_INVAL;
         }
-        newFile = 1;
+        baseAddress = MapViewOfFileEx( hm,
+                                       FILE_MAP_ALL_ACCESS,
+                                       0,
+                                       0,
+                                       (SIZE_T)mappingSize,
+                                       baseAddress );
+        if (baseAddress == NULL) {
+            afsi_log("Error mapping view of file: %d", GetLastError());
+            baseAddress = MapViewOfFile( hm,
+                                         FILE_MAP_ALL_ACCESS,
+                                         0,
+                                         0,
+                                         (SIZE_T)mappingSize);
+            if (baseAddress == NULL) {
+                if (hf != INVALID_HANDLE_VALUE)
+                    CloseHandle(hf);
+                CloseHandle(hm);
+                return CM_ERROR_INVAL;
+            }
+            newCache = 1;
+        }
+        CloseHandle(hm);
+        hMemoryMappedFile = hf;
     }
-    CloseHandle(hm);
 
     config_data_p = (cm_config_data_t *) baseAddress;
 
-    if (!newFile) {
+    if (!newCache) {
         afsi_log("Reusing existing AFS Cache data:");
-        cm_data = *config_data_p;      
+        cm_data = *config_data_p;
 
-	afsi_log("  Base Address   = %p",baseAddress);
+	afsi_log("  Map Address    = %p", baseAddress);
+	afsi_log("  baseAddress    = %p", config_data_p->baseAddress);
 	afsi_log("  stats          = %u", config_data_p->stats);
 	afsi_log("  chunkSize      = %u", config_data_p->chunkSize);
 	afsi_log("  blockSize      = %u", config_data_p->blockSize);
@@ -835,14 +858,23 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
 	afsi_log("  currentSCaches = %u", config_data_p->currentSCaches);
 	afsi_log("  maxSCaches     = %u", config_data_p->maxSCaches);
 
-        // perform validation of persisted data structures
-        // if there is a failure, start from scratch
-        if (cm_ValidateCache && !cm_IsCacheValid()) {
-            newFile = 1;
+        /*
+         * perform validation of persisted data structures
+         * if there is a failure, start from scratch
+         *
+         * if the baseAddress changed then the embedded pointers
+         * within the data structures are no longer valid.
+         * in theory we could walk the tree and adjust the pointer
+         * values based on the offet but that has not been
+         * implemented.
+         */
+        if (baseAddress != cm_data.baseAddress ||
+            cm_ValidateCache && !cm_IsCacheValid()) {
+            newCache = 1;
         }
     }
 
-    if ( newFile ) {
+    if ( newCache ) {
         afsi_log("Building AFS Cache from scratch");
 	memset(&cm_data, 0, sizeof(cm_config_data_t));
         cm_data.size = sizeof(cm_config_data_t);
@@ -866,7 +898,7 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
         cm_data.buf_blockSize = blockSize;
         cm_data.buf_hashSize = osi_PrimeLessThan((afs_uint32)(cacheBlocks/7 + 1));
 
-        cm_data.mountRootGen = time(NULL);
+        cm_data.mountRootGen = 0;
 
         baseAddress += ComputeSizeOfConfigData();
         cm_data.volumeBaseAddress = (cm_volume_t *) baseAddress;
@@ -904,10 +936,18 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
         cm_data.bufEndOfData = (char *) baseAddress;
 	cm_data.buf_dirtyListp = NULL;
 	cm_data.buf_dirtyListEndp = NULL;
-        cm_data.fakeDirVersion = 0x8;
+        /* Make sure the fakeDirVersion is always increasing */
+        cm_data.fakeDirVersion = time(NULL);
+        cm_data.fakeUnique = 0;
         UuidCreate((UUID *)&cm_data.Uuid);
 	cm_data.volSerialNumber = volumeSerialNumber;
 	memcpy(cm_data.Sid, machineSid, sizeof(machineSid));
+
+        /*
+         * make sure that the file is fully allocated
+         * by writing a non-zero byte to the end
+         */
+        cm_data.baseAddress[mappingSize-1] = 0xFF;
     } else {
 	int gennew = 0;
 
@@ -957,24 +997,23 @@ cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chu
     RpcStringFree(&p);
 
     afsi_log("Initializing Volume Data");
-    cm_InitVolume(newFile, maxVols);
+    cm_InitVolume(newCache, maxVols);
 
     afsi_log("Initializing Cell Data");
-    cm_InitCell(newFile, maxCells);
+    cm_InitCell(newCache, maxCells);
 
     afsi_log("Initializing ACL Data");
-    cm_InitACLCache(newFile, 2*stats);
+    cm_InitACLCache(newCache, 2*stats);
 
     afsi_log("Initializing Stat Data");
-    cm_InitSCache(newFile, stats);
-        
+    cm_InitSCache(newCache, stats);
+
     afsi_log("Initializing Data Buffers");
-    cm_InitDCache(newFile, 0, cacheBlocks);
+    cm_InitDCache(newCache, 0, cacheBlocks);
 
     *config_data_p = cm_data;
     config_data_p->dirty = 1;
-    
-    hMemoryMappedFile = hf;
+
     afsi_log("Cache Initialization Complete");
     return 0;
 }
