@@ -1,14 +1,14 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-#ifndef __CM_UTILS_H_ENV__
-#define __CM_UTILS_H_ENV__ 1
+#ifndef OPENAFS_WINNT_AFSD_CM_UTILS_H
+#define OPENAFS_WINNT_AFSD_CM_UTILS_H 1
 
 #define CM_UTILS_SPACESIZE		8192	/* space to allocate */
 typedef struct cm_space {
@@ -79,10 +79,75 @@ extern void cm_Gen8Dot3NameInt(const fschar_t * longname, cm_dirFid_t * pfid,
 extern void cm_Gen8Dot3NameIntW(const clientchar_t* longname, cm_dirFid_t * pfid,
                                 clientchar_t *shortName, clientchar_t **shortNameEndp);
 
+extern void cm_Gen8Dot3VolNameW(afs_uint32 cell, afs_uint32 volume,
+                                clientchar_t *shortName, clientchar_t **shortNameEndp);
+
 extern int cm_MatchMask(clientchar_t *namep, clientchar_t *maskp, int flags);
 
 extern BOOL cm_TargetPerceivedAsDirectory(const fschar_t *target);
 
 extern HANDLE cm_LoadAfsdHookLib(void);
 
-#endif /*  __CM_UTILS_H_ENV__ */
+extern BOOL cm_GetOSFileVersion(char *filename, LARGE_INTEGER *liVer);
+
+extern BOOL msftSMBRedirectorSupportsExtendedTimeouts(void);
+
+/* thread priority */
+extern void cm_UpdateServerPriority(void);
+
+extern void cm_SetRequestStartTime(void);
+
+extern void cm_ResetServerPriority(void);
+
+/* time related functions */
+
+extern void cm_LargeSearchTimeFromUnixTime(FILETIME *largeTimep, time_t unixTime);
+
+extern void cm_UnixTimeFromLargeSearchTime(time_t *unixTimep, FILETIME *largeTimep);
+
+extern void cm_SearchTimeFromUnixTime(afs_uint32 *searchTimep, time_t unixTime);
+
+extern void cm_UnixTimeFromSearchTime(time_t *unixTimep, afs_uint32 searchTime);
+
+extern void cm_utilsInit(void);
+
+extern void cm_utilsCleanup(void);
+
+__inline void
+cm_InterlockedAnd(LONG * pdest, LONG value)
+{
+    LONG orig, current, new;
+
+    current = *pdest;
+
+    do
+    {
+        orig = current;
+        new = orig & value;
+        current = _InterlockedCompareExchange(pdest, new, orig);
+    } while (orig != current);
+}
+
+__inline void
+cm_InterlockedOr(LONG * pdest, LONG value)
+{
+    LONG orig, current, new;
+
+    current = *pdest;
+
+    do
+    {
+        orig = current;
+        new = orig | value;
+        current = _InterlockedCompareExchange(pdest, new, orig);
+    } while (orig != current);
+}
+
+#ifdef DEBUG
+#ifdef _M_IX86
+#define _InterlockedOr   cm_InterlockedOr
+#define _InterlockedAnd  cm_InterlockedAnd
+#endif
+#endif
+
+#endif /*  OPENAFS_WINNT_AFSD_CM_UTILS_H */
