@@ -22,6 +22,11 @@
  * SOFTWARE.
  */
 
+
+#include <afsconfig.h>
+#include <afs/param.h>
+#include <roken.h>
+
 #include<windows.h>
 #include <wchar.h>
 #define _CRT_RAND_S
@@ -455,7 +460,7 @@ NET_API_STATUS NetrShareEnum(
 {
     cm_direnum_t *enump = NULL;
     cm_direnum_entry_t * entryp = NULL;
-    cm_scache_t * dscp = cm_data.rootSCachep;
+    cm_scache_t * dscp;
     cm_user_t *userp = MSRPC_GetCmUser();
     cm_req_t      req;
     int           stopnow = 0;
@@ -470,6 +475,8 @@ NET_API_STATUS NetrShareEnum(
 
     cm_InitReq(&req);
 
+    dscp = cm_RootSCachep(userp, &req);
+
     RPC_SRVSVC_ShareEnumAgeCheck();
 
     /* We only support one server name so ignore 'ServerName'. */
@@ -518,7 +525,8 @@ NET_API_STATUS NetrShareEnum(
     if (ResumeHandle == NULL || *ResumeHandle == 0) {
         cm_dirOp_t    dirop;
 
-        code = cm_BeginDirOp(dscp, userp, &req, CM_DIRLOCK_READ, &dirop);
+        code = cm_BeginDirOp(dscp, userp, &req, CM_DIRLOCK_READ,
+                             CM_DIROP_FLAG_NONE, &dirop);
         if (code == 0) {
             code = cm_BPlusDirEnumerate(dscp, userp, &req, TRUE, NULL, TRUE, &enump);
             if (code) {
@@ -710,7 +718,7 @@ NET_API_STATUS NetrShareGetInfo(
     /* [in] */ DWORD Level,
     /* [switch_is][out] */ LPSHARE_INFO InfoStruct)
 {
-    cm_scache_t * dscp = cm_data.rootSCachep;
+    cm_scache_t * dscp;
     cm_scache_t * scp = NULL;
     cm_user_t *userp = MSRPC_GetCmUser();
     cm_req_t      req;
@@ -725,6 +733,8 @@ NET_API_STATUS NetrShareGetInfo(
 
     cm_InitReq(&req);
 
+    dscp = cm_RootSCachep(userp, &req);
+
     /* Allocate the memory for the response */
     switch (Level) {
     case 2:
@@ -830,9 +840,11 @@ NET_API_STATUS NetrShareGetInfo(
 
             if (cblen) {
                 code = cm_NameI(dscp, pathstr, CM_FLAG_FOLLOW, userp, NULL, &req, &scp);
-                if (code && code != CM_ERROR_NOACCESS)
+                if (code == CM_ERROR_NOSUCHFILE ||
+                    code == CM_ERROR_NOSUCHPATH ||
+                    code == CM_ERROR_BPLUS_NOMATCH)
                     code = cm_NameI(dscp, pathstr, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
-                                      userp, NULL, &req, &scp);
+                                    userp, NULL, &req, &scp);
             }
         }
     }
