@@ -108,6 +108,8 @@ AFSCleanup( IN PDEVICE_OBJECT LibDeviceObject,
 
         stFileCleanup.ProcessId = (ULONGLONG)PsGetCurrentProcessId();
 
+        stFileCleanup.Identifier = (ULONGLONG)pFileObject;
+
         //
         // Perform the cleanup functionality depending on the type of node it is
         //
@@ -358,6 +360,12 @@ AFSCleanup( IN PDEVICE_OBJECT LibDeviceObject,
 
                     ulNotificationFlags |= AFS_REQUEST_FLAG_FILE_DELETED;
 
+                    //
+                    // Indicate the file access mode that is being released
+                    //
+
+                    stFileCleanup.FileAccess = pCcb->FileAccess;
+
                     //
                     // Push the request to the service
                     //
@@ -417,11 +425,26 @@ AFSCleanup( IN PDEVICE_OBJECT LibDeviceObject,
                             AFSAcquireExcl( pObjectInfo->ParentObjectInformation->Specific.Directory.DirectoryNodeHdr.TreeLock,
                                             TRUE);
 
+                            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+                                          AFS_TRACE_LEVEL_VERBOSE,
+                                          "AFSCleanup DE %p for %wZ removing entry\n",
+                                          pCcb->DirectoryCB,
+                                          &pCcb->DirectoryCB->NameInformation.FileName);
+
                             AFSRemoveNameEntry( pObjectInfo->ParentObjectInformation,
                                                 pCcb->DirectoryCB);
 
                             AFSReleaseResource( pObjectInfo->ParentObjectInformation->Specific.Directory.DirectoryNodeHdr.TreeLock);
                         }
+                        else
+                        {
+
+                            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+                                          AFS_TRACE_LEVEL_VERBOSE,
+                                          "AFSCleanup DE %p for %wZ NOT removing entry due to flag set\n",
+                                          pCcb->DirectoryCB,
+                                          &pCcb->DirectoryCB->NameInformation.FileName);
+                        }
                     }
                 }
                 else
@@ -505,6 +528,12 @@ AFSCleanup( IN PDEVICE_OBJECT LibDeviceObject,
                         ulNotificationFlags |= AFS_REQUEST_FLAG_FLUSH_FILE;
                     }
 
+                    //
+                    // Indicate the file access mode that is being released
+                    //
+
+                    stFileCleanup.FileAccess = pCcb->FileAccess;
+
                     //
                     // Push the request to the service
                     //
@@ -667,6 +696,12 @@ AFSCleanup( IN PDEVICE_OBJECT LibDeviceObject,
 
                     ulNotificationFlags |= AFS_REQUEST_FLAG_FILE_DELETED;
 
+                    //
+                    // Indicate the file access mode that is being released
+                    //
+
+                    stFileCleanup.FileAccess = pCcb->FileAccess;
+
                     //
                     // Push the request to the service
                     //
@@ -731,6 +766,15 @@ AFSCleanup( IN PDEVICE_OBJECT LibDeviceObject,
 
                             AFSReleaseResource( pObjectInfo->ParentObjectInformation->Specific.Directory.DirectoryNodeHdr.TreeLock);
                         }
+                        else
+                        {
+
+                            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+                                          AFS_TRACE_LEVEL_VERBOSE,
+                                          "AFSCleanup DE %p for %wZ NOT removing entry due to flag set\n",
+                                          pCcb->DirectoryCB,
+                                          &pCcb->DirectoryCB->NameInformation.FileName);
+                        }
                     }
                 }
 
@@ -761,6 +805,12 @@ AFSCleanup( IN PDEVICE_OBJECT LibDeviceObject,
                         }
                     }
 
+                    //
+                    // Indicate the file access mode that is being released
+                    //
+
+                    stFileCleanup.FileAccess = pCcb->FileAccess;
+
                     AFSProcessRequest( AFS_REQUEST_TYPE_CLEANUP_PROCESSING,
                                        ulNotificationFlags | AFS_REQUEST_FLAG_SYNCHRONOUS,
                                        &pFcb->AuthGroup,
@@ -911,6 +961,12 @@ AFSCleanup( IN PDEVICE_OBJECT LibDeviceObject,
 
                     ulNotificationFlags |= AFS_REQUEST_FLAG_FILE_DELETED;
 
+                    //
+                    // Indicate the file access mode that is being released
+                    //
+
+                    stFileCleanup.FileAccess = pCcb->FileAccess;
+
                     //
                     // Push the request to the service
                     //
@@ -975,6 +1031,15 @@ AFSCleanup( IN PDEVICE_OBJECT LibDeviceObject,
 
                             AFSReleaseResource( pObjectInfo->ParentObjectInformation->Specific.Directory.DirectoryNodeHdr.TreeLock);
                         }
+                        else
+                        {
+
+                            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+                                          AFS_TRACE_LEVEL_VERBOSE,
+                                          "AFSCleanup DE %p for %wZ NOT removing entry due to flag set\n",
+                                          pCcb->DirectoryCB,
+                                          &pCcb->DirectoryCB->NameInformation.FileName);
+                        }
                     }
                 }
 
@@ -1005,6 +1070,12 @@ AFSCleanup( IN PDEVICE_OBJECT LibDeviceObject,
                         }
                     }
 
+                    //
+                    // Indicate the file access mode that is being released
+                    //
+
+                    stFileCleanup.FileAccess = pCcb->FileAccess;
+
                     AFSProcessRequest( AFS_REQUEST_TYPE_CLEANUP_PROCESSING,
                                        ulNotificationFlags | AFS_REQUEST_FLAG_SYNCHRONOUS,
                                        &pFcb->AuthGroup,
