@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -11,19 +11,19 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-RCSID
-    ("$Header: /cvs/openafs/src/butc/butc_xbsa.c,v 1.5 2003/07/15 23:14:48 shadow Exp $");
-
-#include <sys/types.h>
 #include <afs/stds.h>
-#include <stdio.h>
+
+#include <roken.h>
+
 #if defined(AFS_AIX_ENV) || defined(AFS_SUN5_ENV)
 #include <dlfcn.h>
 #endif
-#include <errno.h>
+
 #include "butc_xbsa.h"
 #include <afs/butx.h>
+#include <afs/bubasics.h>
+
+#include "error_macros.h"
 
 extern int debugLevel;
 
@@ -31,6 +31,22 @@ extern int debugLevel;
 char resourceType[20] = "LFS FILE SYSTEM";
 #define GOODSTR(s) ((s)?(s):"<NULL>")
 
+#ifdef NEW_XBSA
+BSA_Int16 (*XBSAInit)(long *, SecurityToken *, ObjectOwner *, char **);
+BSA_Int16 (*XBSABeginTxn)(long);
+BSA_Int16 (*XBSAEndTxn)(long, Vote);
+BSA_Int16 (*XBSATerminate)(long);
+BSA_Int16 (*XBSAQueryObject)(long, QueryDescriptor *, ObjectDescriptor *);
+BSA_Int16 (*XBSAGetObject)(long, ObjectDescriptor *, DataBlock *);
+BSA_Int16 (*XBSAEndData)(long);
+BSA_Int16 (*XBSACreateObject)(long, ObjectDescriptor *, DataBlock *);
+BSA_Int16 (*XBSADeleteObject)(long, CopyType, ObjectName *, CopyId *);
+BSA_Int16 (*XBSAMarkObjectInactive)(long, ObjectName *);
+BSA_Int16 (*XBSASendData)(long, DataBlock *);
+BSA_Int16 (*XBSAGetData)(long, DataBlock *);
+void (*XBSAQueryApiVersion)(ApiVersion *);
+BSA_Int16 (*XBSAGetEnvironment)(long, ObjectOwner *, char **);
+#else
 BSA_Int16(*XBSAInit) (BSA_UInt32 *, SecurityToken *, ObjectOwner *, char **);
 BSA_Int16(*XBSABeginTxn) (BSA_UInt32);
 BSA_Int16(*XBSAEndTxn) (BSA_UInt32, Vote);
@@ -46,7 +62,7 @@ BSA_Int16(*XBSASendData) (BSA_UInt32, DataBlock *);
 BSA_Int16(*XBSAGetData) (BSA_UInt32, DataBlock *);
 BSA_Int16(*XBSAQueryApiVersion) (ApiVersion *);
 BSA_Int16(*XBSAGetEnvironment) (BSA_UInt32, ObjectOwner *, char **);
-
+#endif
 
 xbsa_error(int rc, struct butx_transactionInfo *info)
 {
@@ -119,6 +135,7 @@ xbsa_MountLibrary(struct butx_transactionInfo *info, afs_int32 serverType)
 
     switch (serverType) {
     case XBSA_SERVER_TYPE_ADSM:
+#ifndef NEW_XBSA
 #if defined(AFS_AIX_ENV)
 	dynlib =
 	    dlopen("/usr/lib/libXApi.a(bsashr10.o)",
@@ -129,9 +146,11 @@ xbsa_MountLibrary(struct butx_transactionInfo *info, afs_int32 serverType)
 		       RTLD_NOW | RTLD_LOCAL | RTLD_MEMBER);
 	}
 #elif defined(AFS_SUN5_ENV)
-	dynlib = dlopen("/usr/lib/libXApi.so", RTLD_NOW | RTLD_LOCAL);
+	dlopen ("/usr/lib/libCstd.so.1", RTLD_NOW | RTLD_GLOBAL);
+	dynlib = dlopen("/usr/lib/libXApi.so", RTLD_NOW | RTLD_GLOBAL);
 #else
 	dynlib = NULL;
+#endif
 #endif
 	break;
     default:
@@ -141,15 +160,18 @@ xbsa_MountLibrary(struct butx_transactionInfo *info, afs_int32 serverType)
 	break;
     }
 
+#ifndef NEW_XBSA
     if (dynlib == NULL) {
 	ELog(0,
 	     "xbsa_MountLibrary: The dlopen call to load the XBSA shared library failed\n");
 	return (BUTX_NOLIBRARY);
     }
+#endif
 
     memset(info, 0, sizeof(struct butx_transactionInfo));
     XBSA_SET_SERVER_TYPE(info->serverType, serverType);
 
+#ifndef NEW_XBSA
 #if defined(AFS_AIX_ENV) || defined(AFS_SUN5_ENV)
     XBSAInit = (BSA_Int16(*)
 		(BSA_UInt32 *, SecurityToken *, ObjectOwner *,
@@ -200,16 +222,46 @@ xbsa_MountLibrary(struct butx_transactionInfo *info, afs_int32 serverType)
 	     "xbsa_MountLibrary: The dlopen call to load the XBSA shared library failed\n");
 	return (BUTX_NOLIBRARY);
     }
-
     XBSAQueryApiVersion(&(info->apiVersion));
 #endif
+#else
+#if defined(AFS_AIX_ENV) || defined(AFS_SUN5_ENV) || defined(AFS_LINUX26_ENV)
+    XBSAInit                    = BSAInit;
+    XBSABeginTxn                = BSABeginTxn;
+    XBSAEndTxn                  = BSAEndTxn;
+    XBSATerminate               = BSATerminate;
+    XBSAQueryObject             = BSAQueryObject;
+    XBSAGetObject               = BSAGetObject;
+    XBSAEndData                 = BSAEndData;
+    XBSACreateObject            = BSACreateObject;
+    XBSAMarkObjectInactive      = BSAMarkObjectInactive;
+    XBSADeleteObject            = BSADeleteObject;
+    XBSASendData                = BSASendData;
+    XBSAGetData                 = BSAGetData;
+    XBSAQueryApiVersion         = BSAQueryApiVersion;
+    XBSAGetEnvironment          = BSAGetEnvironment;
 
+    if (!XBSAInit || !XBSABeginTxn || !XBSAEndTxn || !XBSATerminate ||
+	!XBSAQueryObject || !XBSAGetObject || !XBSAEndData ||
+	!XBSACreateObject || !XBSADeleteObject || !XBSAMarkObjectInactive ||
+	!XBSASendData || !XBSAGetData || !XBSAQueryApiVersion ||
+	!XBSAGetEnvironment) {
+	ELog(0,"xbsa_MountLibrary: The dlopen call to load the XBSA shared library failed\n");
+	return(BUTX_NOLIBRARY);
+    }
+    XBSAQueryApiVersion(&(info->apiVersion));
+#endif
+#endif
+
+#ifdef DEBUG_BUTC
+	printf("xbsa_MountLibrary : XBSA function Pointers initialised. \n");
+#endif
     /*
      * Verify the API version
      */
     if ((info->apiVersion.version == XBSA_TS_VERSION)
 	&& (info->apiVersion.release == XBSA_TS_RELEASE)) {
-	/* XOPEN Techincal Standard Level! 
+	/* XOPEN Techincal Standard Level!
 	 * We are coded to the Preliminary Spec!  Time to go boom!
 	 */
 	ELog(0,
@@ -256,8 +308,8 @@ xbsa_Initialize(struct butx_transactionInfo * info, char *bsaObjectOwner,
 {
     char envStrs[XBSA_NUM_ENV_STRS][BSA_MAX_DESC];
     char *envP[XBSA_NUM_ENV_STRS + 1];
-    char *ADSMMaxObject = "MAXOBJ=";
-    char *ADSMServer = "DSMSRVR=";
+    char *ADSMMaxObject = "TSMMAXOBJ=";
+    char *ADSMServer = "TSMSRVR=";
     char *tempStrPtr;
     int i;
     int rc;
@@ -303,6 +355,7 @@ xbsa_Initialize(struct butx_transactionInfo * info, char *bsaObjectOwner,
 	    tempStrPtr = tempStrPtr + strlen(ADSMServer);
 	    strcat(tempStrPtr, serverName);
 	    envP[1] = NULL;
+	    envP[0] = NULL;     /* Hack for TSM V5 */
 	} else {
 	    envP[0] = NULL;
 	    ELog(0, "xbsa_Initialize: The serverName was not specified\n");
@@ -374,6 +427,8 @@ xbsa_Initialize(struct butx_transactionInfo * info, char *bsaObjectOwner,
 	return (BUTX_GETENVFAIL);
     }
 
+ info->maxObjects = 255; /* Hack for ADSM V5: unclear what this actually means... */
+
     switch (XBSA_GET_SERVER_TYPE(info->serverType)) {
     case XBSA_SERVER_TYPE_ADSM:
 	for (i = 0; i < XBSA_NUM_ENV_STRS; i++) {
