@@ -10,18 +10,14 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/cmd/test/dtest.c,v 1.6 2003/07/15 23:14:52 shadow Exp $");
+#include <roken.h>
 
-#include "cmd.h"
-#include <stdio.h>
+#include <afs/cmd.h>
 
-static
-cproc(as, arock)
-     char *arock;
-     struct cmd_syndesc *as;
+static int
+cproc(struct cmd_syndesc *as, void *arock)
 {
-    register struct cmd_item *ti;
+    struct cmd_item *ti;
     printf("in the pear command\n");
     printf("number is %s\n", as->parms[0].items->data);
     if (as->parms[1].items)
@@ -32,11 +28,10 @@ cproc(as, arock)
     return 0;
 }
 
-main(argc, argv)
-     int argc;
-     char **argv;
+int
+main(int argc, char **argv)
 {
-    register struct cmd_syndesc *ts;
+    struct cmd_syndesc *ts;
 
     ts = cmd_CreateSyntax(NULL, cproc, NULL, "describe pear");
     cmd_AddParm(ts, "-num", CMD_SINGLE, 0, "number of pears");
