@@ -1,12 +1,18 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+/*
+ * Do not put anything in this file that relies on Autoconf defines, since
+ * we're not guaranteed to have included afsconfig.h before this header file.
+ * This is an installed header file, and afsconfig.h is not.
+ */
+
 #ifndef OPENAFS_AFS_CONFIG_STDS_H
 #define OPENAFS_AFS_CONFIG_STDS_H	1
 
@@ -45,13 +51,14 @@ pragma Off(Prototype_override_warnings);
 #endif */
 
 #define MAX_AFS_INT32 0x7FFFFFFF
+#define MIN_AFS_INT32 (-MAX_AFS_INT32 - 1)
 #define MAX_AFS_UINT32 0xFFFFFFFF
 #define MAX_AFS_INT64 0x7FFFFFFFFFFFFFFFL
+#define MIN_AFS_INT64 (-MAX_AFS_INT64 - 1)
 #define MAX_AFS_UINT64 0xFFFFFFFFFFFFFFFFL
 
 typedef short afs_int16;
 typedef unsigned short afs_uint16;
-#ifdef  AFS_64BIT_ENV
 typedef int afs_int32;
 typedef unsigned int afs_uint32;
 #if defined(AFS_NT40_ENV) && defined(_MSC_VER)
@@ -62,7 +69,7 @@ typedef long long afs_int64;
 typedef unsigned long long afs_uint64;
 #endif
 #define ZeroInt64(a)       (a = 0)
-#define AssignInt64(a, b) *(b) = (a) 
+#define AssignInt64(a, b) *(b) = (a)
 #define IncInt64(a) (*(a))++
 #define IncUInt64(a) (*(a))++
 #define DecInt64(a) (*(a))--
@@ -83,46 +90,6 @@ typedef unsigned long long afs_uint64;
 #define SplitInt64(t,h,l) (h) = ((afs_int64)t) >> 32; (l) = (t) & MAX_AFS_UINT32;
 #define RoundInt64ToInt32(a)    (a > MAX_AFS_UINT32) ? MAX_AFS_UINT32 : a;
 #define RoundInt64ToInt31(a)    (a > MAX_AFS_INT32) ? MAX_AFS_INT32 : a;
-#else /* AFS_64BIT_ENV */
-typedef long afs_int32;
-typedef unsigned long afs_uint32;
-
-struct Int64 {
-    afs_int32 high;
-    afs_uint32 low;
-};
-typedef struct Int64 afs_int64;
-
-struct u_Int64 {
-    afs_uint32 high;
-    afs_uint32 low;
-};
-typedef struct u_Int64 afs_uint64;
-#define ZeroInt64(a) ((a).high = (a).low = 0)
-#define AssignInt64(a, b) (b)->high = (a).high; (b)->low = (a).low
-#define IncInt64(a) ((++((a)->low)) ? 0 : (a)->high++ )
-#define IncUInt64(a) ((++((a)->low)) ? 0 : (a)->high++ )
-#define DecInt64(a) (((a)->low)-- ? 0 : (a)->high-- )
-#define DecUInt64(a) (((a)->low)-- ? 0 : (a)->high-- )
-#define GTInt64(a,b) (((a).high > (b).high) || (((a).high == (b).high) && ((a).low > (b).low)))
-#define GEInt64(a,b) (((a).high > (b).high) || (((a).high == (b).high) && ((a).low >= (b).low)))
-#define LEInt64(a,b) (((a).high < (b).high) || (((a).high == (b).high) && ((a).low <= (b).low)))
-#define LTInt64(a,b) (((a).high < (b).high) || (((a).high == (b).high) && ((a).low < (b).low)))
-#define CompareInt64(a,b) (((afs_int32)(a).high - (afs_int32)(b).high) || (((a).high == (b).high) && ((a).low - (b).low))) 
-#define AddInt64(a, b, c) {  afs_int64 _a, _b; _a = a; _b = b; (c)->low = _a.low + _b.low; (c)->high = _a.high + _b.high + ((c)->low < _b.low); } 
-#define SubtractInt64(a, b, c) { afs_int64 _a, _b; _a = a; _b = b; (c)->low = _a.low - _b.low;  (c)->high = _a.high - _b.high - (_a.low < _b.low); } 
-#define CompareUInt64(a,b) (((afs_uint32)(a).high - (afs_uint32)(b).high) || (((a).high == (b).high) && ((a).low - (b).low))) 
-#define AddUInt64(a, b, c) {  afs_uint64 _a, _b; _a = a; _b = b; (c)->low = _a.low + _b.low; (c)->high = _a.high + _b.high + ((c)->low < _b.low); } 
-#define SubtractUInt64(a, b, c) { afs_uint64 _a, _b; _a = a; _b = b; (c)->low = _a.low - _b.low;  (c)->high = _a.high - _b.high - (_a.low < _b.low); } 
-#define NonZeroInt64(a)   (a).low || (a).high
-#define Int64ToInt32(a)    (a).low
-#define FillInt64(t,h,l) (t).high = (h); (t).low = (l);
-#define SplitInt64(t,h,l) (h) = (t).high; (l) = (t).low;
-#define RoundInt64ToInt32(a)    (a.high > 0) ? MAX_AFS_UINT32 : a.low;
-#define RoundInt64ToInt31(a)    (a.high > 0) ? MAX_AFS_INT32 : a.low;
-#endif /* AFS_64BIT_ENV */
-
-/* AFS_64BIT_CLIENT should presently be set only for AFS_64BIT_ENV systems */
 
 #ifdef AFS_64BIT_CLIENT
 typedef afs_int64 afs_size_t;
@@ -132,17 +99,10 @@ typedef afs_int32 afs_size_t;
 typedef afs_uint32 afs_offs_t;
 #endif /* AFS_64BIT_CLIENT */
 
-#ifdef AFS_LARGEFILE_ENV
 typedef afs_int64 afs_foff_t;
 typedef afs_uint64 afs_fsize_t;
 typedef afs_int64 afs_sfsize_t;
 #define SplitOffsetOrSize(t,h,l) SplitInt64(t,h,l)
-#else /* !AFS_LARGEFILE_ENV */
-typedef afs_int32 afs_foff_t;
-typedef afs_uint32 afs_fsize_t;
-typedef afs_int32 afs_sfsize_t;
-#define SplitOffsetOrSize(t,h,l) (h) = 0; (l) = (t);
-#endif /* !AFS_LARGEFILE_ENV */
 
 /* Maximum integer sizes.  Also what is expected by %lld, %llu in
  * afs_snprintf. */
@@ -166,7 +126,7 @@ typedef afs_uint32 afs_uintmax_t;
  * some assistence in this matter.  The hyper type is supposed to be compatible
  * with the afsHyper type: the same macros will work on both. */
 
-#if	defined(AFS_64BIT_ENV) && 0
+#if 0
 
 typedef unsigned long afs_hyper_t;
 
@@ -188,7 +148,7 @@ typedef unsigned long afs_hyper_t;
 #define	hadd32(a,b)	((a) += (b))
 #define hshlft(a,n)     ((a)<<(n))
 
-#else /* AFS_64BIT_ENV */
+#else /* 0*/
 
 typedef struct afs_hyper_t {	/* unsigned 64 bit integers */
     unsigned int high;
@@ -231,15 +191,15 @@ typedef struct afs_hyper_t {	/* unsigned 64 bit integers */
 
 #define SIGN 0x80000000
 #define hadd32(a,i) \
-    (((((a).low ^ (int)(i)) & SIGN) \
+    ((void)((((a).low ^ (int)(i)) & SIGN) \
       ? (((((a).low + (int)(i)) & SIGN) == 0) && (a).high++) \
       : (((a).low & (int)(i) & SIGN) && (a).high++)), \
      (a).low += (int)(i))
 
 #define hadd(a,b) (hadd32(a,(b).low), (a).high += (b).high)
-#endif /* AFS_64BIT_ENV */
+#endif /* 0 */
 
-#ifndef	KERNEL
+#if !defined(KERNEL) || defined(UKERNEL)
 #ifndef AFS_NT40_ENV
 #define max(a, b)               ((a) < (b) ? (b) : (a))
 #define min(a, b)               ((a) > (b) ? (b) : (a))
@@ -276,12 +236,77 @@ typedef struct afsUUID afsUUID;
 #define AFS_DEMAND_ATTACH_FS 1
 #endif
 
-#ifdef AFS_HPUX_ENV
-#define static_inline static __inline
-#elif defined(AFS_AIX_ENV) || defined(AFS_SGI_ENV)
+/* A macro that can be used when printf'ing 64 bit integers, as Unix and
+ * windows use a different format string
+ */
+#ifdef AFS_NT40_ENV
+# define AFS_INT64_FMT "I64d"
+# define AFS_UINT64_FMT "I64u"
+# define AFS_PTR_FMT   "p"
+# define AFS_SIZET_FMT "Iu"
+#else
+# define AFS_INT64_FMT "lld"
+# define AFS_UINT64_FMT "llu"
+# define AFS_PTR_FMT   "p"
+# ifdef PRINTF_TAKES_Z_LEN
+#  define AFS_SIZET_FMT "zu"
+# else
+#  define AFS_SIZET_FMT "lu"
+# endif /* PRINTF_TAKES_Z_LEN */
+#endif /* AFS_NT40_ENV */
+
+/* Functions to safely cast afs_int32 and afs_uint32 so they can be used in
+ * printf statemements with %ld and %lu
+ */
+#ifdef AFS_NT40_ENV
+#define static_inline __inline static
+#define hdr_static_inline(x) __inline static x
+#elif defined(AFS_HPUX_ENV) || defined(AFS_USR_HPUX_ENV)
+/* The HPUX compiler can segfault on 'static __inline', so fall back to
+ * just 'static' so we can at least compile */
 #define static_inline static
+#define hdr_static_inline(x) static x
+#elif defined(AFS_AIX_ENV) || defined(AFS_USR_AIX_ENV)
+#define static_inline static
+#define hdr_static_inline(x) static x
+#elif defined(AFS_SGI_ENV) || defined(AFS_USR_SGI_ENV)
+#define static_inline static
+#define hdr_static_inline(x) x
+#elif defined(AFS_NBSD_ENV) && !defined(AFS_NBSD50_ENV)
+#define static_inline static __inline __attribute__((always_inline))
+#define hdr_static_inline(x) static __inline __attribute__((always_inline)) x
 #else
 #define static_inline static inline
+#define hdr_static_inline(x) static inline x
+#endif
+
+hdr_static_inline(long) afs_printable_int32_ld(afs_int32 d) { return (long) d; }
+
+hdr_static_inline(unsigned long) afs_printable_uint32_lu(afs_uint32 d) { return (unsigned long) d; }
+
+#ifdef AFS_64BITUSERPOINTER_ENV
+#define afs_pointer_to_int(p)      ((afs_uint32)  (afs_uint64) (p))
+#define afs_int_to_pointer(i)     ((void *) (afs_uint64) (i))
+#else
+#define afs_pointer_to_int(p)      ((afs_uint32)   (p))
+#define afs_int_to_pointer(i)      ((void *)  (i))
+#endif
+
+#if defined(__GNUC__) && __GNUC__ > 2
+#define AFS_UNUSED __attribute__((unused))
+#define AFS_ATTRIBUTE_FORMAT(style,x,y) __attribute__((format(style, x, y)))
+#define AFS_NORETURN __attribute__((__noreturn__))
+#define AFS_NONNULL(x) __attribute__((__nonnull__ x))
+#elif defined (__clang__)
+#define AFS_UNUSED __attribute__((unused))
+#define AFS_ATTRIBUTE_FORMAT(style,x,y) __attribute__((format(style, x, y)))
+#define AFS_NORETURN __attribute__((__noreturn__))
+#define AFS_NONNULL(x) __attribute__((__nonnull__ x))
+#else
+#define AFS_UNUSED
+#define AFS_ATTRIBUTE_FORMAT(style,x,y)
+#define AFS_NORETURN
+#define AFS_NONNULL(x)
 #endif
 
 #endif /* OPENAFS_CONFIG_AFS_STDS_H */
