@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -9,43 +9,41 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
 #include <afs/stds.h>
-#include <errno.h>
-#include "kauth.h"
-#include <sys/types.h>
-#include <time.h>
-#ifdef AFS_NT40_ENV
-#include <afs/afsutil.h>
-#else
-#include <sys/resource.h>
-#include <sys/file.h>
+
+#include <roken.h>
+#include <afs/opr.h>
+
+#ifdef HAVE_SYS_RESOURCE_H
+# include <sys/resource.h>
 #endif
-#include <stdio.h>
+
+#ifdef IGNORE_SOME_GCC_WARNINGS
+# pragma GCC diagnostic warning "-Wdeprecated-declarations"
+#endif
+
+#include "kauth.h"
+
+#define HC_DEPRECATED
+#include <hcrypto/des.h>
+
 #include <lock.h>
 #include <ubik.h>
 #include <lwp.h>
-#include <des.h>
-#include <des_prototypes.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <rx/rxkad.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <netinet/in.h>
-#endif
-#include <string.h>
 #include <afs/cellconfig.h>
 #include <afs/auth.h>
 #include <afs/com_err.h>
+#include <afs/afsutil.h>
+#include <afs/audit.h>
+
 #include "kautils.h"
 #include "kaserver.h"
 #include "kalog.h"
 #include "kaport.h"
 #include "kauth_internal.h"
-#include "afs/audit.h"
 
 #include "kadatabase.h"
 #include "kaprocs.h"
@@ -75,11 +73,11 @@ extern int npwSums;
 static afs_int32 autoCPWInterval;
 static afs_int32 autoCPWUpdates;
 
-static afs_int32 set_password(struct ubik_trans *tt, char *name, 
+static afs_int32 set_password(struct ubik_trans *tt, char *name,
 			      char *instance,
 			      struct ktc_encryptionKey *password,
 			      afs_int32 kvno, afs_int32 caller);
-static afs_int32 impose_reuse_limits(EncryptionKey *password, 
+static afs_int32 impose_reuse_limits(EncryptionKey *password,
 				     struct kaentry *tentry);
 static int create_user(struct ubik_trans *tt, char *name, char *instance,
 		       struct ktc_encryptionKey *key, afs_int32 caller,
@@ -113,7 +111,7 @@ get_time(Date *timeP,
     int i;
     afs_int32 to;
 
-    gettimeofday(&time, 0);
+    gettimeofday(&time, NULL);
     bit = (random_value[3] >> 31) & 1;	/* get high bit of high word */
     for (i = 0; i < 4; i++) {
 	nbit = random_value[i] >> 31;
@@ -149,12 +147,12 @@ get_time(Date *timeP,
 	if (to) {		/* check if auto cpw is disabled */
 	    if (!(ntohl(tentry.flags) & KAFNOCPW)) {
 		memcpy(&key, &random_value[0], sizeof(key));
-		des_fixup_key_parity(ktc_to_cblock(&key));
+		DES_set_odd_parity(ktc_to_cblock(&key));
 		code =
 		    set_password(tt, KA_ADMIN_NAME, KA_ADMIN_INST, &key, 0,
 				 0);
 		if (code == 0) {
-		    des_init_random_number_generator(ktc_to_cblock(&key));
+		    DES_init_random_number_generator(ktc_to_cblock(&key));
 		    ka_ConvertBytes(buf, sizeof(buf), (char *)&key,
 				    sizeof(key));
 		    es_Report("New Admin key is %s\n", buf);
@@ -173,7 +171,7 @@ get_time(Date *timeP,
 	if (to) {		/* check if auto cpw is disabled */
 	    if (!(ntohl(tentry.flags) & KAFNOCPW)) {
 		memcpy(&key, &random_value[2], sizeof(key));
-		des_fixup_key_parity(ktc_to_cblock(&key));
+		DES_set_odd_parity(ktc_to_cblock(&key));
 		code = set_password(tt, KA_TGS_NAME, lrealm, &key, 0, 0);
 		if (code == 0) {
 		    ka_ConvertBytes(buf, sizeof(buf), (char *)&key,
@@ -219,15 +217,15 @@ initialize_database(struct ubik_trans *tt)
     struct ktc_encryptionKey key;
     int code;
 
-    gettimeofday((struct timeval *)&key, 0);	/* this is just a cheap seed key */
-    des_fixup_key_parity(ktc_to_cblock(&key));
-    des_init_random_number_generator(ktc_to_cblock(&key));
-    if ((code = des_random_key(ktc_to_cblock(&key)))
+    gettimeofday((struct timeval *)&key, NULL);	/* this is just a cheap seed key */
+    DES_set_odd_parity(ktc_to_cblock(&key));
+    DES_init_random_number_generator(ktc_to_cblock(&key));
+    if ((code = DES_new_random_key(ktc_to_cblock(&key)))
 	|| (code =
 	    create_user(tt, KA_ADMIN_NAME, KA_ADMIN_INST, &key, 0,
 			KAFNORMAL | KAFNOSEAL | KAFNOTGS)))
 	return code;
-    if ((code = des_random_key(ktc_to_cblock(&key)))
+    if ((code = DES_new_random_key(ktc_to_cblock(&key)))
 	|| (code =
 	    create_user(tt, KA_TGS_NAME, lrealm, &key, 0,
 			KAFNORMAL | KAFNOSEAL | KAFNOTGS)))
@@ -291,7 +289,7 @@ init_kaprocs(const char *lclpath, int initFlags)
 	     code);
 	return code;
     }
-    des_init_random_number_generator(ktc_to_cblock(&key));
+    DES_init_random_number_generator(ktc_to_cblock(&key));
 
     code = ubik_EndTrans(tt);
     if (code) {
@@ -299,7 +297,7 @@ init_kaprocs(const char *lclpath, int initFlags)
 	return code;
     }
 
-    kaux_opendb(lclpath);	/* aux database stores failure counters */
+    kaux_opendb((char *)lclpath);/* aux database stores failure counters */
     rebuildDatabase = 0;	/* only do this during init */
     kaprocsInited = 1;
     return 0;
@@ -553,7 +551,7 @@ create_user(struct ubik_trans *tt, char *name, char *instance,
 	    struct ktc_encryptionKey *key, afs_int32 caller,
 	    afs_int32 flags)
 {
-    register int code;
+    int code;
     afs_int32 to;
     struct kaentry tentry;
     afs_int32 maxLifetime;
@@ -621,15 +619,16 @@ SKAM_CreateUser(struct rx_call *call, char *aname, char *ainstance,
 
 
 afs_int32
-kamCreateUser(struct rx_call *call, char *aname, char *ainstance, 
+kamCreateUser(struct rx_call *call, char *aname, char *ainstance,
 	      EncryptionKey ainitpw)
 {
-    register int code;
+    int code;
     struct ubik_trans *tt;
     afs_int32 caller;		/* Disk offset of caller's entry */
 
     COUNT_REQ(CreateUser);
-    if (!des_check_key_parity(EncryptionKey_to_cblock(&ainitpw)) || des_is_weak_key(EncryptionKey_to_cblock(&ainitpw)))
+    if (!DES_check_key_parity(EncryptionKey_to_cblock(&ainitpw)) ||
+	DES_is_weak_key(EncryptionKey_to_cblock(&ainitpw)))
 	return KABADKEY;
     if (!name_instance_legal(aname, ainstance))
 	return KABADNAME;
@@ -666,10 +665,10 @@ SKAA_ChangePassword(struct rx_call *call, char *aname, char *ainstance,
 }
 
 afs_int32
-ChangePassWord(struct rx_call *call, char *aname, char *ainstance, 
+ChangePassWord(struct rx_call *call, char *aname, char *ainstance,
 	       ka_CBS *arequest, ka_BBS *oanswer)
 {
-    register int code;
+    int code;
     struct ubik_trans *tt;
     afs_int32 to;		/* offset of block */
     struct kaentry tentry;
@@ -677,7 +676,7 @@ ChangePassWord(struct rx_call *call, char *aname, char *ainstance,
     char *answer;		/* where answer is to be put */
     int answer_len;		/* length of answer packet */
     afs_int32 kvno;		/* requested key version number */
-    des_key_schedule user_schedule;	/* key schedule for user's key */
+    DES_key_schedule user_schedule;	/* key schedule for user's key */
     Date request_time;		/* time request originated */
 
     COUNT_REQ(ChangePassword);
@@ -702,10 +701,10 @@ ChangePassWord(struct rx_call *call, char *aname, char *ainstance,
     }
 
     /* decrypt request w/ user password */
-    if ((code = des_key_sched(ktc_to_cblock(&tentry.key), user_schedule)))
+    if ((code = DES_key_sched(ktc_to_cblock(&tentry.key), &user_schedule)))
 	es_Report("In KAChangePassword: key_sched returned %d\n", code);
-    des_pcbc_encrypt(arequest->SeqBody, &request,
-		     min(arequest->SeqLen, sizeof(request)), user_schedule,
+    DES_pcbc_encrypt(arequest->SeqBody, &request,
+		     min(arequest->SeqLen, sizeof(request)), &user_schedule,
 		     ktc_to_cblockptr(&tentry.key), DECRYPT);
 
     /* validate the request */
@@ -737,8 +736,8 @@ ChangePassWord(struct rx_call *call, char *aname, char *ainstance,
     answer += sizeof(Date);
     memcpy(answer, KA_CPW_ANS_LABEL, KA_LABELSIZE);
 
-    des_pcbc_encrypt(oanswer->SeqBody, oanswer->SeqBody, answer_len,
-		     user_schedule, ktc_to_cblockptr(&tentry.key), ENCRYPT);
+    DES_pcbc_encrypt(oanswer->SeqBody, oanswer->SeqBody, answer_len,
+		     &user_schedule, ktc_to_cblockptr(&tentry.key), ENCRYPT);
 
     code = set_password(tt, aname, ainstance, &request.newpw, kvno, 0);
     if (code) {
@@ -801,7 +800,7 @@ impose_reuse_limits(EncryptionKey *password, struct kaentry *tentry)
 
 
 static afs_int32
-set_password(struct ubik_trans *tt, char *name, char *instance, 
+set_password(struct ubik_trans *tt, char *name, char *instance,
 	     struct ktc_encryptionKey *password, afs_int32 kvno, afs_int32 caller)
 {
     afs_int32 code;
@@ -881,7 +880,7 @@ afs_int32
 kamSetPassword(struct rx_call *call, char *aname, char *ainstance,
 	       afs_int32 akvno, EncryptionKey apassword)
 {
-    register int code;
+    int code;
     struct ubik_trans *tt;
     afs_int32 caller;		/* Disk offset of caller's entry */
     struct kaentry tentry;
@@ -889,7 +888,8 @@ kamSetPassword(struct rx_call *call, char *aname, char *ainstance,
     COUNT_REQ(SetPassword);
     if (akvno > MAXKAKVNO)
 	return KABADARGUMENT;
-    if (!des_check_key_parity(EncryptionKey_to_cblock(&apassword)) || des_is_weak_key(EncryptionKey_to_cblock(&apassword)))
+    if (!DES_check_key_parity(EncryptionKey_to_cblock(&apassword)) ||
+	DES_is_weak_key(EncryptionKey_to_cblock(&apassword)))
 	return KABADKEY;
 
     if (!name_instance_legal(aname, ainstance))
@@ -980,7 +980,7 @@ GetEndTime(Date start,		/* start time of ticket */
 static afs_int32
 PrepareTicketAnswer(ka_BBS *oanswer, afs_int32 challenge, char *ticket,
 		    afs_int32 ticketLen, struct ktc_encryptionKey *sessionKey,
-		    Date start, Date end, struct kaentry *caller, 
+		    Date start, Date end, struct kaentry *caller,
 		    struct kaentry *server, char *cell, char *label)
 {
     afs_int32 code;
@@ -1042,7 +1042,7 @@ PrepareTicketAnswer(ka_BBS *oanswer, afs_int32 challenge, char *ticket,
    is normally disabled for these two principals. */
 
 static afs_int32
-Authenticate(int version, struct rx_call *call, char *aname, char *ainstance, 
+Authenticate(int version, struct rx_call *call, char *aname, char *ainstance,
 	     Date start, Date end, ka_CBS *arequest, ka_BBS *oanswer)
 {
     int code;
@@ -1061,7 +1061,7 @@ Authenticate(int version, struct rx_call *call, char *aname, char *ainstance,
     int answer_len;		/* length of answer packet */
     Date answer_time;		/* 1+ request time in network order */
     afs_int32 temp;		/* for htonl conversions */
-    des_key_schedule user_schedule;	/* key schedule for user's key */
+    DES_key_schedule user_schedule;	/* key schedule for user's key */
     afs_int32 tgskvno;		/* key version of service key */
     struct ktc_encryptionKey tgskey;	/* service key for encrypting ticket */
     Date now;
@@ -1086,7 +1086,7 @@ Authenticate(int version, struct rx_call *call, char *aname, char *ainstance,
     }
 #ifdef LOCKPW
     /* have to check for locked before verifying the password, otherwise all
-     * KALOCKED means is "yup, you guessed the password all right, now wait a 
+     * KALOCKED means is "yup, you guessed the password all right, now wait a
      * few minutes and we'll let you in"
      */
     if (kaux_islocked
@@ -1100,10 +1100,10 @@ Authenticate(int version, struct rx_call *call, char *aname, char *ainstance,
     save_principal(authPrincipal, aname, ainstance, 0);
 
     /* decrypt request w/ user password */
-    if ((code = des_key_sched(ktc_to_cblock(&tentry.key), user_schedule)))
+    if ((code = DES_key_sched(ktc_to_cblock(&tentry.key), &user_schedule)))
 	es_Report("In KAAuthenticate: key_sched returned %d\n", code);
-    des_pcbc_encrypt(arequest->SeqBody, &request,
-		     min(arequest->SeqLen, sizeof(request)), user_schedule,
+    DES_pcbc_encrypt(arequest->SeqBody, &request,
+		     min(arequest->SeqLen, sizeof(request)), &user_schedule,
 		     ktc_to_cblockptr(&tentry.key), DECRYPT);
 
     request.time = ntohl(request.time);	/* reorder date */
@@ -1166,7 +1166,7 @@ Authenticate(int version, struct rx_call *call, char *aname, char *ainstance,
     tgskvno = ntohl(server.key_version);
     memcpy(&tgskey, &server.key, sizeof(tgskey));
 
-    code = des_random_key(ktc_to_cblock(&sessionKey));
+    code = DES_new_random_key(ktc_to_cblock(&sessionKey));
     if (code) {
 	code = KANOKEYS;
 	goto abort;
@@ -1243,8 +1243,8 @@ Authenticate(int version, struct rx_call *call, char *aname, char *ainstance,
 	code = KAINTERNALERROR;
 	goto abort;
     }
-    des_pcbc_encrypt(oanswer->SeqBody, oanswer->SeqBody, oanswer->SeqLen,
-		     user_schedule, ktc_to_cblockptr(&tentry.key), ENCRYPT);
+    DES_pcbc_encrypt(oanswer->SeqBody, oanswer->SeqBody, oanswer->SeqLen,
+		     &user_schedule, ktc_to_cblockptr(&tentry.key), ENCRYPT);
     code = ubik_EndTrans(tt);
     KALOG(aname, ainstance, sname, sinst, NULL, call->conn->peer->host,
 	  LOG_AUTHENTICATE);
@@ -1260,7 +1260,7 @@ Authenticate(int version, struct rx_call *call, char *aname, char *ainstance,
 
 afs_int32
 SKAA_Authenticate_old(struct rx_call *call, char *aname, char *ainstance,
-		      Date start, Date end, ka_CBS *arequest, 
+		      Date start, Date end, ka_CBS *arequest,
 		      ka_BBS *oanswer)
 {
     int code;
@@ -1276,7 +1276,7 @@ SKAA_Authenticate_old(struct rx_call *call, char *aname, char *ainstance,
 }
 
 afs_int32
-SKAA_Authenticate(struct rx_call *call, char *aname, char *ainstance, 
+SKAA_Authenticate(struct rx_call *call, char *aname, char *ainstance,
 		  Date start, Date end, ka_CBS *arequest, ka_BBS *oanswer)
 {
     int code;
@@ -1291,7 +1291,7 @@ SKAA_Authenticate(struct rx_call *call, char *aname, char *ainstance,
 }
 
 afs_int32
-SKAA_AuthenticateV2(struct rx_call *call, char *aname, char *ainstance, 
+SKAA_AuthenticateV2(struct rx_call *call, char *aname, char *ainstance,
 		    Date start, Date end, ka_CBS *arequest, ka_BBS *oanswer)
 {
     int code;
@@ -1402,13 +1402,13 @@ kamSetFields(struct rx_call *call,
 	tentry.max_ticket_lifetime = htonl(alifetime);
 
 #ifndef NOPWCONTROLS
-    /* 
+    /*
      * We've packed a bunch of bytes into a long for backward compatibility.
      * These include password expiration time, and some failed login limits
      * counters.  Now let's unpack them and stick them into the
      * kaentry struct.  All the bytes have values in the range
      * 1..255, else they were not specified in the interface, and are
-     * set to zero. 
+     * set to zero.
      * In the case of password expiration times, 1 means password never
      * expires (==>0), 2 means password only lives for one day (==>1),
      * and so on.
@@ -1481,12 +1481,12 @@ SKAM_DeleteUser(struct rx_call *call, char *aname, char *ainstance)
 afs_int32
 kamDeleteUser(struct rx_call *call, char *aname, char *ainstance)
 {
-    register int code;
+    int code;
     struct ubik_trans *tt;
     afs_int32 caller;
     afs_int32 to;
     struct kaentry tentry;
-    int nfailures;
+    unsigned int nfailures;
     afs_uint32 locktime;
 
     COUNT_REQ(DeleteUser);
@@ -1562,7 +1562,7 @@ kamGetEntry(struct rx_call *call,
 	    afs_int32 aversion,		/* major version assumed by caller */
 	    kaentryinfo *aentry)	/* entry data copied here */
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     afs_int32 callerIndex;
     struct kaentry caller;
@@ -1627,7 +1627,7 @@ kamGetEntry(struct rx_call *call,
 
     temp = pack_long(tentry.misc_auth_bytes);
     aentry->misc_auth_bytes = temp;
-    /* 
+    /*
      * only return user's key if security disabled or if admin and
      * we have an encrypted connection to the user
      */
@@ -1688,7 +1688,7 @@ kamListEntry(struct rx_call *call,
 	     afs_int32 *count,		/* total entries in database */
 	     kaident *name)		/* name & instance of this entry */
 {
-    register int code;
+    int code;
     struct ubik_trans *tt;
     afs_int32 caller;
     struct kaentry tentry;
@@ -1739,7 +1739,7 @@ GetTicket(int version,
     int import, export;
     struct ubik_trans *tt;
     struct ktc_encryptionKey tgskey;
-    des_key_schedule schedule;
+    DES_key_schedule schedule;
     afs_int32 to;
     char name[MAXKTCNAMELEN];
     char instance[MAXKTCNAMELEN];
@@ -1800,7 +1800,7 @@ GetTicket(int version,
 	    code = KANOAUTH;
 	goto abort;
     }
-    code = des_key_sched(ktc_to_cblock(&authSessionKey), schedule);
+    code = DES_key_sched(ktc_to_cblock(&authSessionKey), &schedule);
     if (code) {
 	code = KANOAUTH;
 	goto abort;
@@ -1818,7 +1818,7 @@ GetTicket(int version,
 	goto abort;
     }
 
-    des_ecb_encrypt(atimes->SeqBody, &times, schedule, DECRYPT);
+    DES_ecb_encrypt((DES_cblock *)atimes->SeqBody, (DES_cblock *)&times, &schedule, DECRYPT);
     times.start = ntohl(times.start);
     times.end = ntohl(times.end);
     code = tkt_CheckTimes(times.start, times.end, now);
@@ -1855,7 +1855,7 @@ GetTicket(int version,
     }
     save_principal(tgsServerPrincipal, sname, sinstance, 0);
 
-    code = des_random_key(ktc_to_cblock(&sessionKey));
+    code = DES_new_random_key(ktc_to_cblock(&sessionKey));
     if (code) {
 	code = KANOKEYS;
 	goto abort;
@@ -1928,8 +1928,8 @@ GetTicket(int version,
 	code = KAINTERNALERROR;
 	goto abort;
     }
-    des_pcbc_encrypt(oanswer->SeqBody, oanswer->SeqBody, oanswer->SeqLen,
-		     schedule, ktc_to_cblockptr(&authSessionKey), ENCRYPT);
+    DES_pcbc_encrypt(oanswer->SeqBody, oanswer->SeqBody, oanswer->SeqLen,
+		     &schedule, ktc_to_cblockptr(&authSessionKey), ENCRYPT);
     code = ubik_EndTrans(tt);
     KALOG(name, instance, sname, sinstance, (import ? authDomain : NULL),
 	  call->conn->peer->host, LOG_GETTICKET);
@@ -1984,7 +1984,7 @@ SKAT_GetTicket(struct rx_call *call,
 }
 
 afs_int32
-SKAM_GetStats(struct rx_call *call, afs_int32 version, 
+SKAM_GetStats(struct rx_call *call, afs_int32 version,
 	      afs_int32 *admin_accounts, kasstats *statics,
 	      kadstats *dynamics)
 {
@@ -1996,7 +1996,7 @@ SKAM_GetStats(struct rx_call *call, afs_int32 version,
 }
 
 afs_int32
-kamGetStats(struct rx_call *call, afs_int32 version, 
+kamGetStats(struct rx_call *call, afs_int32 version,
 	    afs_int32 *admin_accounts, kasstats *statics,
 	    kadstats *dynamics)
 {
@@ -2083,8 +2083,7 @@ afs_int32
 kamGetPassword(struct rx_call *call, char *name, EncryptionKey *password)
 {
     int code = KANOAUTH;
-    COUNT_REQ(GetPassword);
-
+    AFS_UNUSED COUNT_REQ(GetPassword);
 #ifdef GETPASSWORD
     {
 	afs_int32 to;
@@ -2142,10 +2141,10 @@ kamGetRandomKey(struct rx_call *call, EncryptionKey *key)
 {
     int code;
 
-    COUNT_REQ(GetRandomKey);
+    AFS_UNUSED COUNT_REQ(GetRandomKey);
     if ((code = AwaitInitialization()))
 	return code;
-    code = des_random_key(EncryptionKey_to_cblock(key));
+    code = DES_new_random_key(EncryptionKey_to_cblock(key));
     if (code)
 	return KANOKEYS;
     return 0;
@@ -2224,7 +2223,7 @@ kamDebug(struct rx_call *call,
     return 0;
 }
 
-/* these are auxiliary routines. They don't do any Ubik stuff.  They use 
+/* these are auxiliary routines. They don't do any Ubik stuff.  They use
  * a tacked-on-the-side data file.
  * prob'ly ought to check the noauth flag.
  */
@@ -2233,12 +2232,12 @@ afs_int32
 SKAM_Unlock(struct rx_call *call,
 	    char *aname,
 	    char *ainstance,
-	    afs_int32 spare1, 
-	    afs_int32 spare2, 
-	    afs_int32 spare3, 
+	    afs_int32 spare1,
+	    afs_int32 spare2,
+	    afs_int32 spare3,
 	    afs_int32 spare4)
 {
-    register int code;
+    int code;
     struct ubik_trans *tt;
     afs_int32 caller;
     afs_int32 to;
@@ -2278,12 +2277,12 @@ SKAM_LockStatus(struct rx_call *call,
 		char *aname,
 		char *ainstance,
 		afs_int32 *lockeduntil,
-		afs_int32 spare1, 
-		afs_int32 spare2, 
-		afs_int32 spare3, 
+		afs_int32 spare1,
+		afs_int32 spare2,
+		afs_int32 spare3,
 		afs_int32 spare4)
 {
-    register int code;
+    int code;
     struct ubik_trans *tt;
     afs_int32 callerIndex;
     afs_int32 to;
