@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -26,12 +26,12 @@
  *                      is a member of (since the entries field is used to
  *                      list it's members). This new field is supergroups and
  *                      has two entries. If more are required, a continuation
- *                      record is formed. 
+ *                      record is formed.
  *                      There are two additional fields required, nextsg is
  *                      an address of the next continuation record for this
- *                      group, and countsg is the count for the number of 
+ *                      group, and countsg is the count for the number of
  *                      groups this group is a member of.
- *                   
+ *
  *
  *
  *      09/18/95 jjm    Add mdw's changes to afs-3.3a Changes:
@@ -110,39 +110,32 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
 #include <afs/stds.h>
-#ifdef	AFS_AIX32_ENV
-#include <signal.h>
-#endif
-#include <sys/types.h>
-#include <stdio.h>
-#include <fcntl.h>
-#include <sys/stat.h>
+
+#include <roken.h>
+#include <afs/opr.h>
+
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
 #include <WINNT/afsevent.h>
-#else
-#include <netdb.h>
-#include <netinet/in.h>
 #endif
-#include <string.h>
+
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <rx/rx_globals.h>
+#include <rx/rxstat.h>
 #include <lock.h>
 #include <ubik.h>
+#include <afs/cmd.h>
 #include <afs/cellconfig.h>
 #include <afs/auth.h>
 #include <afs/keys.h>
+#include <afs/afsutil.h>
+#include <afs/audit.h>
+#include <afs/com_err.h>
+
 #include "ptserver.h"
 #include "ptprototypes.h"
 #include "error_macros.h"
-#include "afs/audit.h"
-#include <afs/afsutil.h>
-#include <afs/com_err.h>
-#include <rx/rxstat.h>
 
 /* make	all of these into a structure if you want */
 struct prheader cheader;
@@ -153,10 +146,8 @@ struct afsconf_dir *prdir;
 extern afs_int32 depthsg;
 #endif
 
-int pr_realmNameLen;
 char *pr_realmName;
 
-int debuglevel = 0;
 int restricted = 0;
 int rxMaxMTU = -1;
 int rxBind = 0;
@@ -201,35 +192,53 @@ pr_rxstat_userok(struct rx_call *call)
     return afsconf_SuperUser(prdir, call, NULL);
 }
 
+enum optionsList {
+    OPT_database,
+    OPT_access,
+    OPT_groupdepth,
+    OPT_restricted,
+    OPT_auditlog,
+    OPT_auditiface,
+    OPT_config,
+    OPT_debug,
+    OPT_logfile,
+    OPT_threads,
+    OPT_syslog,
+    OPT_peer,
+    OPT_process,
+    OPT_rxbind,
+    OPT_rxmaxmtu,
+    OPT_dotted
+};
+
 int
 main(int argc, char **argv)
 {
-    register afs_int32 code;
-    afs_int32 myHost;
-    register struct hostent *th;
+    afs_int32 code;
+    afs_uint32 myHost;
+    struct hostent *th;
     char hostname[64];
     struct rx_service *tservice;
-    struct rx_securityClass *sc[3];
-#if 0
-    struct ktc_encryptionKey tkey;
-#endif
-    int kerberosKeys;		/* set if found some keys */
+    struct rx_securityClass **securityClasses;
+    afs_int32 numClasses;
     int lwps = 3;
     char clones[MAXHOSTSPERCELL];
     afs_uint32 host = htonl(INADDR_ANY);
+    struct cmd_syndesc *opts;
+    struct cmd_item *list;
 
-    const char *pr_dbaseName;
+    char *pr_dbaseName;
+    char *configDir;
+    char *logFile;
     char *whoami = "ptserver";
 
-    int a;
-    char arg[100];
-
     char *auditFileName = NULL;
+    char *interface = NULL;
 
 #ifdef	AFS_AIX32_ENV
     /*
-     * The following signal action for AIX is necessary so that in case of a 
-     * crash (i.e. core is generated) we can include the user's data section 
+     * The following signal action for AIX is necessary so that in case of a
+     * crash (i.e. core is generated) we can include the user's data section
      * in the core dump. Unfortunately, by default, only a partial core is
      * generated which, in many cases, isn't too useful.
      */
@@ -254,7 +263,9 @@ main(int argc, char **argv)
 	exit(2);
     }
 
-    pr_dbaseName = AFSDIR_SERVER_PRDB_FILEPATH;
+    pr_dbaseName = strdup(AFSDIR_SERVER_PRDB_FILEPATH);
+    configDir = strdup(AFSDIR_SERVER_ETC_DIRPATH);
+    logFile = strdup(AFSDIR_SERVER_PTLOG_FILEPATH);
 
 #if defined(SUPERGROUPS)
     /* make sure the structures for database records are the same size */
@@ -262,157 +273,144 @@ main(int argc, char **argv)
 	|| (sizeof(struct prentryg) != ENTRYSIZE)) {
 	fprintf(stderr,
 		"The structures for the database records are different"
-		" sizes\n" "struct prentry = %d\n" "struct prentryg = %d\n"
+		" sizes\n" "struct prentry = %" AFS_SIZET_FMT "\n"
+                "struct prentryg = %" AFS_SIZET_FMT "\n"
 		"ENTRYSIZE = %d\n", sizeof(struct prentry),
 		sizeof(struct prentryg), ENTRYSIZE);
 	PT_EXIT(1);
     }
 #endif
 
-    for (a = 1; a < argc; a++) {
-	int alen;
-	lcstring(arg, argv[a], sizeof(arg));
-	alen = strlen(arg);
-	if (strcmp(argv[a], "-d") == 0) {
-	    if ((a + 1) >= argc) {
-		fprintf(stderr, "missing argument for -d\n"); 
-		return -1; 
-	    }
-	    debuglevel = atoi(argv[++a]);
-	    LogLevel = debuglevel;
-	} else if ((strncmp(arg, "-database", alen) == 0)
-	    || (strncmp(arg, "-db", alen) == 0)) {
-	    pr_dbaseName = argv[++a];	/* specify a database */
-	} else if (strncmp(arg, "-p", alen) == 0) {
-	    lwps = atoi(argv[++a]);
-	    if (lwps > 16) {	/* maximum of 16 */
-		printf("Warning: '-p %d' is too big; using %d instead\n",
-		       lwps, 16);
-		lwps = 16;
-	    } else if (lwps < 3) {	/* minimum of 3 */
-		printf("Warning: '-p %d' is too small; using %d instead\n",
-		       lwps, 3);
-		lwps = 3;
-	    }
-	} else if (strncmp(arg, "-rebuild", alen) == 0)	/* rebuildDB++ */
-	    ;
+    cmd_DisableAbbreviations();
+    cmd_DisablePositionalCommands();
+    opts = cmd_CreateSyntax(NULL, NULL, NULL, NULL);
+
+/* ptserver specific options */
+    cmd_AddParmAtOffset(opts, OPT_database, "-database", CMD_SINGLE,
+		        CMD_OPTIONAL, "database file");
+    cmd_AddParmAlias(opts, OPT_database, "db");
+
+    cmd_AddParmAtOffset(opts, OPT_access, "-default_access", CMD_SINGLE,
+		        CMD_OPTIONAL, "default access flags for new entries");
 #if defined(SUPERGROUPS)
-	else if ((strncmp(arg, "-groupdepth", alen) == 0)
-		 || (strncmp(arg, "-depth", alen) == 0)) {
-	    depthsg = atoi(argv[++a]);	/* Max search depth for supergroups */
-	}
+    cmd_AddParmAtOffset(opts, OPT_groupdepth, "-groupdepth", CMD_SINGLE,
+		        CMD_OPTIONAL, "max search depth for supergroups");
+    cmd_AddParmAlias(opts, OPT_groupdepth, "depth");
 #endif
-	else if (strncmp(arg, "-default_access", alen) == 0) {
-	    prp_user_default = prp_access_mask(argv[++a]);
-	    prp_group_default = prp_access_mask(argv[++a]);
-	}
-	else if (strncmp(arg, "-restricted", alen) == 0) {
-	    restricted = 1;
-	}
-	else if (strncmp(arg, "-rxbind", alen) == 0) {
-	    rxBind = 1;
-	}
-	else if (strncmp(arg, "-allow-dotted-principals", alen) == 0) {
-	    rxkadDisableDotCheck = 1;
-	}
-	else if (strncmp(arg, "-enable_peer_stats", alen) == 0) {
-	    rx_enablePeerRPCStats();
-	} else if (strncmp(arg, "-enable_process_stats", alen) == 0) {
-	    rx_enableProcessRPCStats();
-	}
-#ifndef AFS_NT40_ENV
-	else if (strncmp(arg, "-syslog", alen) == 0) {
-	    /* set syslog logging flag */
-	    serverLogSyslog = 1;
-	} else if (strncmp(arg, "-syslog=", MIN(8, alen)) == 0) {
-	    serverLogSyslog = 1;
-	    serverLogSyslogFacility = atoi(arg + 8);
-	}
-#endif
-	else if (strncmp(arg, "-auditlog", alen) == 0) {
-	    auditFileName = argv[++a];
+    cmd_AddParmAtOffset(opts, OPT_restricted, "-restricted", CMD_FLAG,
+		        CMD_OPTIONAL, "enable restricted mode");
 
-	} else if (strncmp(arg, "-audit-interface", alen) == 0) {
-	    char *interface = argv[++a];
-	    if (osi_audit_interface(interface)) {
-		printf("Invalid audit interface '%s'\n", interface);
-		PT_EXIT(1);
-	    }
-	}
-	else if (!strncmp(arg, "-rxmaxmtu", alen)) {
-	    if ((a + 1) >= argc) {
-		fprintf(stderr, "missing argument for -rxmaxmtu\n");
-		PT_EXIT(1);
-	    }
-	    rxMaxMTU = atoi(argv[++a]);
-	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) ||
-		 (rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
-		printf("rxMaxMTU %d invalid; must be between %d-%lu\n",
-			rxMaxMTU, RX_MIN_PACKET_SIZE,
-			RX_MAX_PACKET_DATA_SIZE);
-		PT_EXIT(1);
-	    }
-	} 
-	else if (*arg == '-') {
-	    /* hack in help flag support */
+    /* general server options */
+    cmd_AddParmAtOffset(opts, OPT_auditlog, "-auditlog", CMD_SINGLE,
+		 	CMD_OPTIONAL, "location of audit log");
+    cmd_AddParmAtOffset(opts, OPT_auditiface, "-audit-interface", CMD_SINGLE,
+		        CMD_OPTIONAL, "interface to use for audit logging");
+    cmd_AddParmAtOffset(opts, OPT_config, "-config", CMD_SINGLE,
+		        CMD_OPTIONAL, "configuration location");
+    cmd_AddParmAtOffset(opts, OPT_debug, "-d", CMD_SINGLE,
+		        CMD_OPTIONAL, "debug level");
+    cmd_AddParmAtOffset(opts, OPT_logfile, "-logfile", CMD_SINGLE,
+		        CMD_OPTIONAL, "location of logfile");
+    cmd_AddParmAtOffset(opts, OPT_threads, "-p", CMD_SINGLE,
+		        CMD_OPTIONAL, "number of threads");
+#if !defined(AFS_NT40_ENV)
+    cmd_AddParmAtOffset(opts, OPT_syslog, "-syslog", CMD_SINGLE_OR_FLAG, 
+		        CMD_OPTIONAL, "log to syslog");
+#endif
 
-#if defined(SUPERGROUPS)
-#ifndef AFS_NT40_ENV
-	    printf("Usage: ptserver [-database <db path>] "
-		   "[-auditlog <log path>] "
-		   "[-audit-interface <file|sysvmq> (default is file)] "
-		   "[-syslog[=FACILITY]] [-d <debug level>] "
-		   "[-p <number of processes>] [-rebuild] "
-		   "[-groupdepth <depth>] "
-		   "[-restricted] [-rxmaxmtu <bytes>] [-rxbind] "
-		   "[-allow-dotted-principals] "
-		   "[-enable_peer_stats] [-enable_process_stats] "
-		   "[-default_access default_user_access default_group_access] "
-		   "[-help]\n");
-#else /* AFS_NT40_ENV */
-	    printf("Usage: ptserver [-database <db path>] "
-		   "[-auditlog <log path>] "
-		   "[-audit-interface <file|sysvmq> (default is file)] "
-		   "[-d <debug level>] "
-		   "[-p <number of processes>] [-rebuild] [-rxbind] "
-		   "[-allow-dotted-principals] "
-		   "[-default_access default_user_access default_group_access] "
-		   "[-restricted] [-rxmaxmtu <bytes>] [-rxbind] "
-		   "[-groupdepth <depth>] " "[-help]\n");
-#endif
-#else
-#ifndef AFS_NT40_ENV
-	    printf("Usage: ptserver [-database <db path>] "
-		   "[-auditlog <log path>] "
-		   "[-audit-interface <file|sysvmq> (default is file)] "
-		   "[-d <debug level>] "
-		   "[-syslog[=FACILITY]] "
-		   "[-p <number of processes>] [-rebuild] "
-		   "[-enable_peer_stats] [-enable_process_stats] "
-		   "[-default_access default_user_access default_group_access] "
-		   "[-restricted] [-rxmaxmtu <bytes>] [-rxbind] "
-		   "[-allow-dotted-principals] "
-		   "[-help]\n");
-#else /* AFS_NT40_ENV */
-	    printf("Usage: ptserver [-database <db path>] "
-		   "[-auditlog <log path>] [-d <debug level>] "
-		   "[-default_access default_user_access default_group_access] "
-		   "[-restricted] [-rxmaxmtu <bytes>] [-rxbind] "
-		   "[-allow-dotted-principals] "
-		   "[-p <number of processes>] [-rebuild] " "[-help]\n");
-#endif
-#endif
-	    fflush(stdout);
+    /* rx options */
+    cmd_AddParmAtOffset(opts, OPT_peer, "-enable_peer_stats", CMD_FLAG,
+		        CMD_OPTIONAL, "enable RX transport statistics");
+    cmd_AddParmAtOffset(opts, OPT_process, "-enable_process_stats", CMD_FLAG,
+		        CMD_OPTIONAL, "enable RX RPC statistics");
+    cmd_AddParmAtOffset(opts, OPT_rxbind, "-rxbind", CMD_FLAG,
+		        CMD_OPTIONAL, "bind only to the primary interface");
+    cmd_AddParmAtOffset(opts, OPT_rxmaxmtu, "-rxmaxmtu", CMD_SINGLE,
+		        CMD_OPTIONAL, "maximum MTU for RX");
 
+    /* rxkad options */
+    cmd_AddParmAtOffset(opts, OPT_dotted, "-allow-dotted-principals",
+		        CMD_FLAG, CMD_OPTIONAL,
+		        "permit Kerberos 5 principals with dots");
+
+    code = cmd_Parse(argc, argv, &opts);
+    if (code)
+	PT_EXIT(1);
+
+    if (cmd_OptionAsList(opts, OPT_access, &list) == 0) {
+	prp_user_default = prp_access_mask(list->data);
+	if (list->next == NULL || list->next->data == NULL) {
+	    fprintf(stderr, "Missing second argument for -default_access\n");
 	    PT_EXIT(1);
 	}
-#if defined(SUPERGROUPS)
-	else {
-	    fprintf(stderr, "Unrecognized arg: '%s' ignored!\n", arg);
-	}
-#endif
+	prp_group_default = prp_access_mask(list->next->data);
     }
 
+#if defined(SUPERGROUPS)
+    cmd_OptionAsInt(opts, OPT_groupdepth, &depthsg);
+#endif
+
+    cmd_OptionAsFlag(opts, OPT_restricted, &restricted);
+
+    /* general server options */
+    cmd_OptionAsString(opts, OPT_auditlog, &auditFileName);
+
+    if (cmd_OptionAsString(opts, OPT_auditiface, &interface) == 0) {
+	if (osi_audit_interface(interface)) {
+	    printf("Invalid audit interface '%s'\n", interface);
+	    PT_EXIT(1);
+	}
+	free(interface);
+    }
+
+    cmd_OptionAsString(opts, OPT_config, &configDir);
+    cmd_OptionAsInt(opts, OPT_debug, &LogLevel);
+    cmd_OptionAsString(opts, OPT_database, &pr_dbaseName);
+    cmd_OptionAsString(opts, OPT_logfile, &logFile);
+
+    if (cmd_OptionAsInt(opts, OPT_threads, &lwps) == 0) {
+	if (lwps > 16) {	/* maximum of 16 */
+	    printf("Warning: '-p %d' is too big; using %d instead\n",
+		   lwps, 16);
+	    lwps = 16;
+	} else if (lwps < 3) {	/* minimum of 3 */
+	    printf("Warning: '-p %d' is too small; using %d instead\n",
+		   lwps, 3);
+	    lwps = 3;
+	}
+    }
+
+#ifndef AFS_NT40_ENV
+    if (cmd_OptionPresent(opts, OPT_syslog)) {
+	serverLogSyslog = 1;
+	cmd_OptionAsInt(opts, OPT_syslog, &serverLogSyslogFacility);
+    }
+#endif
+
+    /* rx options */
+    if (cmd_OptionPresent(opts, OPT_peer))
+	rx_enablePeerRPCStats();
+
+    if (cmd_OptionPresent(opts, OPT_process))
+	rx_enableProcessRPCStats();
+
+    cmd_OptionAsFlag(opts, OPT_rxbind, &rxBind);
+
+    if (cmd_OptionAsInt(opts, OPT_rxmaxmtu, &rxMaxMTU) == 0) {
+	if ((rxMaxMTU < RX_MIN_PACKET_SIZE) ||
+	    (rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
+	    printf("rxMaxMTU %d invalid; must be between %d-%" AFS_SIZET_FMT "\n",
+		   rxMaxMTU, RX_MIN_PACKET_SIZE,
+		   RX_MAX_PACKET_DATA_SIZE);
+	    PT_EXIT(1);
+	}
+    }
+
+    /* rxkad options */
+    cmd_OptionAsFlag(opts, OPT_dotted, &rxkadDisableDotCheck);
+
+    cmd_FreeOptions(&opts);
+
     if (auditFileName) {
 	osi_audit_file(auditFileName);
 	osi_audit(PTS_StartEvent, 0, AUD_END);
@@ -421,10 +419,10 @@ main(int argc, char **argv)
 #ifndef AFS_NT40_ENV
     serverLogSyslogTag = "ptserver";
 #endif
-    OpenLog(AFSDIR_SERVER_PTLOG_FILEPATH);	/* set up logging */
+    OpenLog(logFile);	/* set up logging */
     SetupLogSignals();
 
-    prdir = afsconf_Open(AFSDIR_SERVER_ETC_DIRPATH);
+    prdir = afsconf_Open(configDir);
     if (!prdir) {
 	fprintf(stderr, "ptserver: can't open configuration directory.\n");
 	PT_EXIT(1);
@@ -448,45 +446,22 @@ main(int argc, char **argv)
 	fprintf(stderr, "ptserver: couldn't get address of this host.\n");
 	PT_EXIT(1);
     }
-    memcpy(&myHost, th->h_addr, sizeof(afs_int32));
+    memcpy(&myHost, th->h_addr, sizeof(afs_uint32));
 
     /* get list of servers */
     code =
-	afsconf_GetExtendedCellInfo(prdir, NULL, "afsprot", &info, &clones);
+	afsconf_GetExtendedCellInfo(prdir, NULL, "afsprot", &info, clones);
     if (code) {
 	afs_com_err(whoami, code, "Couldn't get server list");
 	PT_EXIT(2);
     }
     pr_realmName = info.name;
-    pr_realmNameLen = strlen(pr_realmName);
 
-#if 0
-    /* get keys */
-    code = afsconf_GetKey(prdir, 999, &tkey);
-    if (code) {
-	afs_com_err(whoami, code,
-		"couldn't get bcrypt keys from key file, ignoring.");
-    }
-#endif
-    {
-	afs_int32 kvno;		/* see if there is a KeyFile here */
-	struct ktc_encryptionKey key;
-	code = afsconf_GetLatestKey(prdir, &kvno, &key);
-	kerberosKeys = (code == 0);
-	if (!kerberosKeys)
-	    printf
-		("ptserver: can't find any Kerberos keys, code = %d, ignoring\n",
-		 code);
-    }
-    if (kerberosKeys) {
-	/* initialize ubik */
-	ubik_CRXSecurityProc = afsconf_ClientAuth;
-	ubik_CRXSecurityRock = prdir;
-	ubik_SRXSecurityProc = afsconf_ServerAuth;
-	ubik_SRXSecurityRock = prdir;
-	ubik_CheckRXSecurityProc = afsconf_CheckAuth;
-	ubik_CheckRXSecurityRock = prdir;
-    }
+    /* initialize ubik */
+    ubik_SetClientSecurityProcs(afsconf_ClientAuth, afsconf_UpToDate, prdir);
+    ubik_SetServerSecurityProcs(afsconf_BuildServerSecurityObjects,
+				afsconf_CheckAuth, prdir);
+
     /* The max needed is when deleting an entry.  A full CoEntry deletion
      * required removal from 39 entries.  Each of which may refers to the entry
      * being deleted in one of its CoEntries.  If a CoEntry is freed its
@@ -499,14 +474,14 @@ main(int argc, char **argv)
 
     if (rxBind) {
 	afs_int32 ccode;
-	if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || 
+	if (AFSDIR_SERVER_NETRESTRICT_FILEPATH ||
 	    AFSDIR_SERVER_NETINFO_FILEPATH) {
 	    char reason[1024];
 	    ccode = parseNetFiles(SHostAddrs, NULL, NULL,
 					   ADDRSPERSITE, reason,
 					   AFSDIR_SERVER_NETINFO_FILEPATH,
 					   AFSDIR_SERVER_NETRESTRICT_FILEPATH);
-	} else 
+	} else
 	{
 	    ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
 	}
@@ -522,7 +497,7 @@ main(int argc, char **argv)
 
     code =
 	ubik_ServerInitByInfo(myHost, htons(AFSCONF_PROTPORT), &info, clones,
-			      (char *)pr_dbaseName, &dbase);
+			      pr_dbaseName, &dbase);
     if (code) {
 	afs_com_err(whoami, code, "Ubik init failed");
 	PT_EXIT(2);
@@ -531,12 +506,7 @@ main(int argc, char **argv)
     pt_hook_write();
 #endif
 
-    sc[0] = rxnull_NewServerSecurityObject();
-    sc[1] = 0;
-    if (kerberosKeys) {
-	sc[2] = rxkad_NewServerSecurityObject(0, prdir, afsconf_GetKey, NULL);
-    } else
-	sc[2] = sc[0];
+    afsconf_BuildServerSecurityObjects(prdir, &securityClasses, &numClasses);
 
     /* Disable jumbograms */
     rx_SetNoJumbo();
@@ -546,8 +516,8 @@ main(int argc, char **argv)
     }
 
     tservice =
-	rx_NewServiceHost(host, 0, PRSRV, "Protection Server", sc, 3,
-		      PR_ExecuteRequest);
+	rx_NewServiceHost(host, 0, PRSRV, "Protection Server", securityClasses,
+		          numClasses, PR_ExecuteRequest);
     if (tservice == (struct rx_service *)0) {
 	fprintf(stderr, "ptserver: Could not create new rx service.\n");
 	PT_EXIT(3);
@@ -560,8 +530,8 @@ main(int argc, char **argv)
     }
 
     tservice =
-	rx_NewServiceHost(host, 0, RX_STATS_SERVICE_ID, "rpcstats", sc, 3,
-		      RXSTATS_ExecuteRequest);
+	rx_NewServiceHost(host, 0, RX_STATS_SERVICE_ID, "rpcstats",
+			  securityClasses, numClasses, RXSTATS_ExecuteRequest);
     if (tservice == (struct rx_service *)0) {
 	fprintf(stderr, "ptserver: Could not create new rx service.\n");
 	PT_EXIT(3);
@@ -572,6 +542,14 @@ main(int argc, char **argv)
     /* allow super users to manage RX statistics */
     rx_SetRxStatUserOk(pr_rxstat_userok);
 
+    LogCommandLine(argc, argv, "ptserver",
+#if defined(SUPERGROUPS)
+		   "1.1",
+#else
+		   "1.0",
+#endif
+		   "Starting AFS", FSLog);
+
     rx_StartServer(1);
     osi_audit(PTS_FinishEvent, -1, AUD_END);
     exit(0);
