@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,11 +10,8 @@
 /* The rxkad security object.  Routines used by both client and servers. */
 
 #include <afsconfig.h>
-#ifdef KERNEL
-#include "afs/param.h"
-#else
 #include <afs/param.h>
-#endif
+#include <afs/stds.h>
 
 #ifdef AFS_SUN59_ENV
 #include <sys/time_impl.h>
@@ -22,48 +19,33 @@
 
 #define INCLUDE_RXKAD_PRIVATE_DECLS
 
-
 #ifdef KERNEL
 #ifndef UKERNEL
-#include "afs/stds.h"
 #include "afs/afs_osi.h"
-#if defined(AFS_AIX_ENV) || defined(AFS_AUX_ENV) || defined(AFS_SUN5_ENV) 
+#if defined(AFS_AIX_ENV) || defined(AFS_AUX_ENV) || defined(AFS_SUN5_ENV)
 #include "h/systm.h"
 #endif
-#if defined(AFS_DARWIN60_ENV) || defined(AFS_OBSD_ENV)
+#if defined(AFS_DARWIN_ENV) || defined(AFS_OBSD_ENV)
 #include "h/kernel.h"
 #endif
 #include "h/types.h"
 #include "h/time.h"
-#ifndef AFS_LINUX22_ENV
-#include "rpc/types.h"
-#include "rx/xdr.h"
-#endif /* AFS_LINUX22_ENV */
 #else /* !UKERNEL */
 #include "afs/sysincludes.h"
 #include "afsincludes.h"
 #endif /* !UKERNEL */
-#include "rx/rx.h"
-
 #else /* KERNEL */
-#include <afs/stds.h>
-#include <sys/types.h>
-#include <string.h>
-#include <time.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#ifdef AFS_PTHREAD_ENV
+#include <roken.h>
+#if defined(AFS_NT40_ENV) && defined(AFS_PTHREAD_ENV)
 #define RXKAD_STATS_DECLSPEC __declspec(dllexport)
 #endif
-#else
-#include <netinet/in.h>
-#endif
-#include <rx/rx.h>
-#include <rx/xdr.h>
 #include <afs/afsutil.h>
 #endif /* KERNEL */
 
-#include <des/stats.h>
+#include <rx/rx.h>
+#include <rx/xdr.h>
+
+#include "stats.h"
 #include "private_data.h"
 #define XPRT_RXKAD_COMMON
 
@@ -81,14 +63,36 @@ struct rxkad_global_stats rxkad_global_stats = { 0 };
 pthread_mutex_t rxkad_global_stats_lock;
 pthread_key_t rxkad_stats_key;
 #else /* AFS_PTHREAD_ENV */
-#if defined(KERNEL) && !defined(UKERNEL)
-struct rxkad_stats rxkad_stats = { { 0 } }; 
-#else
-/* Move delaration of this to des/key_sched.c */
-#endif
+struct rxkad_stats rxkad_stats = { { 0 } };
 #endif /* AFS_PTHREAD_ENV */
 
 #ifdef AFS_PTHREAD_ENV
+/* Pthread initialisation */
+static pthread_once_t rxkad_once_init = PTHREAD_ONCE_INIT;
+extern pthread_mutex_t rxkad_client_uid_mutex;
+extern pthread_mutex_t rxkad_random_mutex;
+
+static void
+rxkad_global_stats_init(void)
+{
+    osi_Assert(pthread_mutex_init(&rxkad_global_stats_lock, (const pthread_mutexattr_t *)0) == 0);
+    osi_Assert(pthread_key_create(&rxkad_stats_key, NULL) == 0);
+    memset(&rxkad_global_stats, 0, sizeof(rxkad_global_stats));
+}
+
+static void
+rxkad_InitPthread(void) {
+    MUTEX_INIT(&rxkad_client_uid_mutex, "uid", MUTEX_DEFAULT, 0);
+    MUTEX_INIT(&rxkad_random_mutex, "rxkad random", MUTEX_DEFAULT, 0);
+
+    rxkad_global_stats_init();
+}
+
+void
+rxkad_Init(void) {
+    osi_Assert(pthread_once(&rxkad_once_init, rxkad_InitPthread) == 0);
+}
+
 /* rxkad_stats related stuff */
 
 /*
@@ -103,20 +107,14 @@ struct rxkad_stats rxkad_stats = { { 0 } };
 	    (ptr)->prev->next = (ptr);		 \
 	else					 \
 	    (head) = (ptr);			 \
-	assert((head) && ((head)->prev == NULL)); \
+	osi_Assert((head) && ((head)->prev == NULL)); \
     } while(0)
 
-void rxkad_global_stats_init(void) {
-    assert(pthread_mutex_init(&rxkad_global_stats_lock, (const pthread_mutexattr_t *)0) == 0);
-    assert(pthread_key_create(&rxkad_stats_key, NULL) == 0);
-    memset(&rxkad_global_stats, 0, sizeof(rxkad_global_stats));
-}
-
-rxkad_stats_t * 
+rxkad_stats_t *
 rxkad_thr_stats_init(void) {
     rxkad_stats_t * rxkad_stats;
     rxkad_stats = (rxkad_stats_t *)malloc(sizeof(rxkad_stats_t));
-    assert(rxkad_stats != NULL && pthread_setspecific(rxkad_stats_key,rxkad_stats) == 0);
+    osi_Assert(rxkad_stats != NULL && pthread_setspecific(rxkad_stats_key,rxkad_stats) == 0);
     memset(rxkad_stats,0,sizeof(rxkad_stats_t));
     RXKAD_GLOBAL_STATS_LOCK;
     DLL_INSERT_TAIL(rxkad_stats, rxkad_global_stats.first, rxkad_global_stats.last, next, prev);
@@ -126,7 +124,7 @@ rxkad_thr_stats_init(void) {
 
 int rxkad_stats_agg(rxkad_stats_t * rxkad_stats) {
     rxkad_stats_t * thr_stats;
-    assert(rxkad_stats != NULL);
+    osi_Assert(rxkad_stats != NULL);
     memset(rxkad_stats, 0, sizeof(rxkad_stats_t));
     RXKAD_GLOBAL_STATS_LOCK;
     for (thr_stats = rxkad_global_stats.first; thr_stats != NULL; thr_stats = thr_stats->next) {
@@ -171,6 +169,8 @@ int rxkad_stats_agg(rxkad_stats_t * rxkad_stats) {
 	rxkad_stats->des_encrypts[1] += thr_stats->des_encrypts[1];
 	rxkad_stats->des_key_scheds += thr_stats->des_key_scheds;
 	rxkad_stats->des_randoms += thr_stats->des_randoms;
+	rxkad_stats->clientObjects += thr_stats->clientObjects;
+	rxkad_stats->serverObjects += thr_stats->serverObjects;
 	rxkad_stats->spares[0] += thr_stats->spares[0];
 	rxkad_stats->spares[1] += thr_stats->spares[1];
 	rxkad_stats->spares[2] += thr_stats->spares[2];
@@ -179,12 +179,16 @@ int rxkad_stats_agg(rxkad_stats_t * rxkad_stats) {
 	rxkad_stats->spares[5] += thr_stats->spares[5];
 	rxkad_stats->spares[6] += thr_stats->spares[6];
 	rxkad_stats->spares[7] += thr_stats->spares[7];
-	rxkad_stats->spares[8] += thr_stats->spares[8];
-	rxkad_stats->spares[9] += thr_stats->spares[9];
     }
     RXKAD_GLOBAL_STATS_UNLOCK;
     return 0;
 }
+#else
+void
+rxkad_Init(void)
+{
+    return;
+}
 #endif /* AFS_PTHREAD_ENV */
 
 /* static prototypes */
@@ -199,7 +203,7 @@ int
 rxkad_SetupEndpoint(struct rx_connection *aconnp,
 		    struct rxkad_endpoint *aendpointp)
 {
-    register afs_int32 i;
+    afs_int32 i;
 
     aendpointp->cuid[0] = htonl(aconnp->epoch);
     i = aconnp->cid & RX_CIDMASK;
@@ -259,7 +263,7 @@ rxkad_SetLevel(struct rx_connection *conn, rxkad_level level)
 	rx_SetSecurityMaxTrailerSize(conn, 4);
     } else if (level == rxkad_crypt) {
 	rx_SetSecurityHeaderSize(conn, 8);
-	rx_SetSecurityMaxTrailerSize(conn, 8);	/* XXX was 7, but why screw with 
+	rx_SetSecurityMaxTrailerSize(conn, 8);	/* XXX was 7, but why screw with
 						 * unaligned accesses? */
     }
 }
@@ -272,7 +276,7 @@ ComputeSum(struct rx_packet *apacket, fc_KeySchedule * aschedule,
 	   afs_int32 * aivec)
 {
     afs_uint32 word[2];
-    register afs_uint32 t;
+    afs_uint32 t;
 
     t = apacket->header.callNumber;
     word[0] = htonl(t);
@@ -337,13 +341,13 @@ rxkad_NewConnection(struct rx_securityClass *aobj,
 
     if (rx_IsServerConn(aconn)) {
 	int size = sizeof(struct rxkad_sconn);
-	aconn->securityData = (char *)rxi_Alloc(size);
+	aconn->securityData = rxi_Alloc(size);
 	memset(aconn->securityData, 0, size);	/* initialize it conveniently */
     } else {			/* client */
 	struct rxkad_cprivate *tcp;
 	struct rxkad_cconn *tccp;
 	int size = sizeof(struct rxkad_cconn);
-	tccp = (struct rxkad_cconn *)rxi_Alloc(size);
+	tccp = rxi_Alloc(size);
 	aconn->securityData = (char *)tccp;
 	memset(aconn->securityData, 0, size);	/* initialize it conveniently */
 	tcp = (struct rxkad_cprivate *)aobj->privateData;
@@ -489,7 +493,7 @@ rxkad_CheckPacket(struct rx_securityClass *aobj, struct rx_call *acall,
 	return RXKADSEALEDINCON;
     nlen = word & 0xffff;	/* get real user data length */
 
-    /* The sealed length should be no larger than the initial length, since the  
+    /* The sealed length should be no larger than the initial length, since the
      * reverse (round-up) occurs in ...PreparePacket */
     if (nlen > len)
 	return RXKADDATALEN;
