@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -9,35 +9,24 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-RCSID
-    ("$Header: /cvs/openafs/src/update/utils.c,v 1.10 2004/06/23 14:27:46 shadow Exp $");
-
 #include <afs/stds.h>
-#include <rx/rxkad.h>
-#include "global.h"
+
+#include <roken.h>
+
 #ifdef AFS_NT40_ENV
 #include <afs/errmap_nt.h>
 #include <afs/afsutil.h>
 #include <WINNT/afssw.h>
 #endif
-#include <stdio.h>
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#include <stdlib.h>
+
+#include <rx/rxkad.h>
+#include "global.h"
+
 
 int
 AddToList(struct filestr **ah, char *aname)
 {
-    register struct filestr *tf;
+    struct filestr *tf;
     tf = (struct filestr *)malloc(sizeof(struct filestr));
     tf->next = *ah;
     *ah = tf;
@@ -49,7 +38,7 @@ AddToList(struct filestr **ah, char *aname)
 int
 ZapList(struct filestr **ah)
 {
-    register struct filestr *tf, *nf;
+    struct filestr *tf, *nf;
     for (tf = *ah; tf; tf = nf) {
 	nf = tf->next;		/* save before freeing */
 	free(tf->name);
